TITLE "Binary searches for an item in an array of items"

.586
.model flat, stdcall	; memory model
option casemap:none	    ; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
; (none)

; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
; (none)


.code
; ----------------------------------------------------------------------------
; prototype:
;    LPVOID __stdcall BinarySearch(
;		 LPVOID pTarget,
;		 LPVOID pArray,
;		 DWORD dwArrayCount,
;		 DWORD dwElemSize
;		 int (__cdecl *pfnCOMPARE)(const void* pElem1, const void* pElem2));
;
; Description:
;   Does a binary search for the Target item. The caller passes in a user-
;   defined __cdecl compare routine to compare specific element against the
;   target item passed in.
;
; Returns:
;		A pointer to the element in array if found otherwise NULL.
; ============================================================================
align 16
BinarySearch proc public uses ebx esi edi pTarget:dword, pArray:dword, dwArrayCount:dword, dwElemSize:dword, pfnCompare:dword
	; set up the registers with our information.
	mov	edi, [dwArrayCount]		; high end
	sub	edi, 1h
	xor	esi, esi						; low end

SearchIt:
	; check our bounds
	cmp	esi, edi
	jg		short NotFound

	; get the mid element index value
	lea	ebx, [edi+esi]
	shr	ebx, 1h

	; get the element information
	mov	eax, [dwElemSize]
	mov	ecx, [pArray]
	imul	eax, ebx
	add	eax, [pArray]

	; call the user-defined compare
	push	eax			; pArray element
	push	[pTarget]	; pTarget
	call	DWORD PTR pfnCompare	; __cdecl call

	; test the result and see if they found match
	test	eax, eax

	; get back our 'mid' value and element offset
	pop	eax
	pop	eax
	
	; branch accordingly
	je		short SearchEnd
	jl		short AdjustUpperHalf

	; move low end bound to search index + 1
	mov	esi, ebx
	add	esi, 1h
	jmp	short SearchIt

AdjustUpperHalf:
	; move high end bound to search index - 1
	mov	edi, ebx
	sub	edi, 1h
	jmp	short SearchIt

NotFound:
	xor	eax, eax

SearchEnd:
	ret
BinarySearch ENDP

end
