TITLE	FileVerInfo.asm

.486
.model flat, stdcall      ; memory model & calling convention
option casemap :none      ; case sensitive


comment #
; ----------------------------------------------------------------------------
; include headers
; ----------------------------------------------------------------------------
include \masm32\include\windows.inc
include \masm32\include\kernel32.inc
include \masm32\include\version.inc


; ----------------------------------------------------------------------------
; const defines
; ----------------------------------------------------------------------------
.const
align 16
	LanguageSubBlock		db	'\VarFileInfo\Translation', 0
	ItemSubBlockFmt		db '\StringFileInfo\%04x%04x\%s', 0
	

; ----------------------------------------------------------------------------
; module code
; ----------------------------------------------------------------------------
.code
align 16


; ----------------------------------------------------------------------------
; prototype:
; LPVOID __stdcall InitStringFileVerInfo(LPCSTR FileName)
; ----------------------------------------------------------------------------
InitStringFileVerInfo proc public uses esi edi FileName:LPCSTR
	LOCAL BufSize:dword

	; set up our basic working variables to be used
	lea eax, [BufSize]
	mov esi, [FileName]	
	xor edi, edi
	
	; get the length of the version information in the FileName specified
	push eax
	push esi
	call GetFileVersionInfoSize
	
	; make sure we found it ok and it has a length
	test eax, eax
	je NoVersionFound
	
	; allocate a buffer on the heap with the size
	mov [BufSize], eax
	push eax
	call GetProcessHeap
	push 0
	push eax
	call HeapAlloc	; we assume success always
	
	; save our buffer pointer
	mov edi, eax
	
	; now, get the actual data from the resource
	push eax
	push [BufSize]
	push 0
	push esi
	call GetFileVersionInfo
	
	; make sure we read it ok
	test eax, eax
	jnz Done
	
	; some error, so clean up our buffer
	call GetProcessHeap
	push edi
	push 0
	push eax
	call HeapFree
	xor edi, edi
	
NoVersionFound:
	mov eax, edi
	
Done:
	ret
InitStringFileVerInfo endp

; ----------------------------------------------------------------------------
; prototype:
; LPCSTR __stdcall GetStringFileVerInfo(LPVOID InfoBlockInst, LPCSTR ItemName) 
; ----------------------------------------------------------------------------
GetStringFileVerInfo proc public InfoBlockInst:LPVOID, ItemName:LPCSTR
	LOCAL lang:dword
	LOCAL len:dword
	LOCAL String[256]:byte
	LOCAL pointer:dword
	
	; get the language and code page we are going to be using	
	lea edx, len
	lea ecx, lang
	lea eax, LanguageSubBlock
	push edx
	push ecx
	push eax
	push [InfoBlockInst]
	call VerQueryValue
	
	; make sure we read it in ok
	test eax, eax
	je	Done
	
	mov eax, 0
	cmp len, 0
	je Done
	
	mov eax, [lang]
	movzx ecx, word ptr [eax].LANGANDCODEPAGE.language
	movzx edx, word ptr [eax].LANGANDCODEPAGE.codepage
	lea eax, ItemSubBlockFmt
	push edx
	push ecx
	lea edx, String
	push eax
	push edx
	call wsprintf
	
	lea edx, len
	lea ecx, pointer
	lea eax, String
	push edx
	push ecx
	push eax
	push [InfoBlockInst]
	call VerQueryValue 	
	
Done:
	ret
GetStringFileVerInfo endp

; ----------------------------------------------------------------------------
; prototype:
; Void __stdcall ExitStringFileVerInfo(LPVOID InfoBlockInst)
; ----------------------------------------------------------------------------
ExitStringFileVerInfo proc public InfoBlockInst:LPVOID
	call GetProcessHeap

	push [InfoBlockInst]
	push 0
	push eax
	call HeapFree

	ret
ExitStringFileVerInfo endp


; ----------------------------------------------------------------------------
#

END
