TITLE "Searches a linear array for an item"

.586
.model flat, stdcall	; memory model
option casemap:none	    ; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
; (none)

; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
; (none)


.code
; ----------------------------------------------------------------------------
; prototype:
;   LPVOID __stdcall LinearSearch(LPVOID pTarget, LPVOID pArray, DWORD dwArrayCount, DWORD dwElemSize, int (__cdecl *pfnCOMPARE)(const void* pElem1, const void* pElem2));
;
; Description:
;   Does a linear search (top to bottom) for Target item.  This function uses
;   a user-defined __cdecl compare routine to compare specified element in
;   array against Target item passed in.
;
; Returns:
;   A pointer to the element in array if found otherwise NULL.
; ----------------------------------------------------------------------------
align 16
LinearSearch proc public uses ebx pTarget:dword, pArray:dword, dwArrayCount:dword, dwElemSize:dword, pfnCompare:dword
; set up EAX register with our array for searching
	mov	eax, [pArray]
	
; set up EBX register with what we are looking for in the array
	mov	ebx, [pTarget]

; set up ECX with the ending array address for our loop
	mov	ecx, [dwArrayCount]
	lea	ecx, [eax + ecx]
	
; set up EDX with the element size to move the array pointer
; when a no-match is found
	mov	edx, [dwElemSize]
	
	
__LinearSearchLoop:
; save our ecx and edx registers because they'll get changed from the call to
; the user-defined compare routine
	push ecx
	push edx

; need to push our pArray address onto the stack to become the 2nd param
; value in the callback function.
	push eax

; need to push our pTarget value onto the stack to become the 1st param
; value in the callback function
	push ebx

; call use user-defined compare function.
	call DWORD PTR pfnCompare		;  __cdecl function

; check the result (will be in EAX).  0 = match, non-zero = no match.
; this will set our flag for jumping.
	test eax, eax

; now that the flag has been set, we can restore our registers and pop
; what we have pushed onto the stack
	pop	ebx
	pop	eax
	pop	edx
	pop	ecx
	
; use our flag to continue or return.  if we are returning from this
; function, EAX will have the element address for the return value
	je	short __LinearSearchEnd

; go to the next element in the array
	add	eax, edx

; see if we have reached the end of the array
	cmp 	eax, ecx
	jne	short __LinearSearchLoop

; we have reached the end -- need to set eax to NULL
	sub	eax, eax

__LinearSearchEnd:
	ret
LinearSearch ENDP

end
