TITLE "Creates a 32-bit HASH value"

.586
.model flat, stdcall	; memory model
option casemap:none	    ; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
; (none)

; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
; (none)


.code
; ----------------------------------------------------------------------------
; prototype:
;   unsigned long __stdcall Hash32(const void* pBuffer, unsigned Len);
;
; Description:
;   Creates a simple 32-bit hash value from a case-sensitive string or binary
;	data.
;
; Returns:
;   A 32-bit hash value.
; ============================================================================
align 16
Hash32 proc public uses esi edi Buffer:dword, Len:dword
	; load our registers with the params
	mov	esi, [Buffer]
	mov	ecx, [Len]

	; start our value out at zero
	xor	eax, eax
	
	
; equivalent c code of this loop
; n = 0;
; while (len--)
;		n = (n << 3) ^ n ^ *pBuffer++;
; n &= 7fffffffH;

align 4
@@:
	movzx	edx, byte ptr [esi+ecx-1h]
	mov	edi, eax
	shl	edi, 3

	xor	edx, edi
	xor	eax, edx

	sub	ecx, 1
	jnz	@B

	and	eax, 07fffffffH

	ret
Hash32 ENDP

END
