TITLE "Get the raw pixel data from a HBITMAP"

.586
.model flat, stdcall      ; memory model & calling convention
option casemap :none      ; case sensitive


; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
include Windows.inc
include Gdi32.inc


; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
include asm_Graphics.inc




.code
; ----------------------------------------------------------------------------
; Prototype		:
;		void __stdcall GetBitmapData(BitmapBitsInfo* pBitmapInfo);
;
; Description	:
;		Gets the bits associated to this HBITMAP handle in the pBits variable.
;		Callee will need to free the pBits variable when done to avoid a memory
;		leak by calling invoke DeleteObject, BitmapBitsInfo->hDibSection 
; ----------------------------------------------------------------------------
align 16
GetBitmapData proc public uses ebx esi edi ptrInfo:dword

	LOCAL hOldDIBBitmap	:dword		; HBITMAP
	LOCAL	hOldDDBBitmap	:dword		; HBITMAP
	LOCAL	bmi				:BITMAPINFO
	
	; start by setting the value of pBits to NULL and zero out the BITMAPINFO struct
	lea	edi, [bmi]
	mov	ebx, [ptrInfo]
	mov	ecx, sizeof BITMAPINFO / 4
	xor	eax, eax
	mov	[ebx].BMBitsData.pBits, eax
	rep	stosd
	
	; get our width and height into registers for transfering into the
	; BITMAPINFO variable
	mov	edx, [ebx].BMBitsData.ptDimensions.y
	mov	ecx, [ebx].BMBitsData.ptDimensions.x
	
	; need to make the height a negative value so we have a TOP-DOWN image
	; instead of a BOTTOM-UP image
	neg	edx
	
	; set the necessary information of the structure to create a proper
	; DIB section.
	mov	[bmi.bmiHeader.biSize], sizeof BITMAPINFO
	mov	[bmi.bmiHeader.biWidth], ecx
	mov	[bmi.bmiHeader.biHeight], edx
	mov	[bmi.bmiHeader.biPlanes], 1
	mov	[bmi.bmiHeader.biBitCount], 020h

	; create two memory DC objects to select the DiBSection bitmap and
	; the passed in bitmap into.
	xor	edi, edi
	invoke CreateCompatibleDC, edi
	mov	esi, eax

	invoke CreateCompatibleDC, edi
	mov	edi, eax

	; select the passed in bitmap object into the device context
	invoke SelectObject, edi, [ebx].BMBitsData.hBitmap
	mov [hOldDDBBitmap], eax

	; create a DIB section bitmap.  we'll be bitblting the DDB into this DIB section
	lea	edx, [ebx].BMBitsData.pBits
	lea	ecx, [bmi]
	invoke CreateDIBSection, esi, ecx, DIB_RGB_COLORS, edx, 0, 0
	mov [ebx].BMBitsData.hDibSection, eax

	; select the DIBSection bitmap object into the device context
	invoke SelectObject, esi, eax
	mov [hOldDIBBitmap], eax

	; transfer the data from the DDB into the DIB so we can access the raw pixels --
	; the raw pixels after the bitblt will be in pBits variable.
	xor ecx, ecx
	invoke BitBlt, esi, ecx, ecx, [ebx].BMBitsData.ptDimensions.x, [ebx].BMBitsData.ptDimensions.y, edi, ecx, ecx, SRCCOPY

	; select the original bitmaps back into the DCs
	invoke SelectObject, esi, [hOldDIBBitmap]	
	invoke SelectObject, edi, [hOldDDBBitmap]

	; destroy the DC objects
	invoke DeleteDC, esi
	invoke DeleteDC, edi

	ret
GetBitmapData endp
 
END
