TITLE "Count the number of unique colors in the image"

.586
.model flat			; memory model
option casemap:none	; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
include Kernel32.inc

; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
; (none)


.code

; ----------------------------------------------------------------------------
; prototype:
;		int __stdcall CountColors(LPVOID pRawPixelData, int Width, int Height);
;
; Description:
;		Returns the count of unique colors in a given image.
;
; Params:
;		- A pointer to the raw pixel data in 32-bit format (ARGB) or (BGRA)
;		- Width of image
;		- Height of image
; ----------------------------------------------------------------------------
align 16
_CountColors@12 PROC NEAR
	push	ecx
	push	ebx
	push	ebp
	push	esi
	push	edi
;	call	DWORD PTR __imp__GetProcessHeap@0
	call	GetProcessHeap
	mov	ebx, eax

	; create our main table array (256 * 256 * 4-bytes)
	mov	esi, [esp+24]
	push	262144
	push	8
	push	eax
;	call  dword ptr __imp__HeapAlloc@12
	call	HeapAlloc

	; figure out the size of the image (width * height * 4-bytes)
	mov	ecx, [esp+28]
	imul	ecx, [esp+32]
	mov	[esp+16], eax
	lea	ebp, [esi+ecx*4]

align 4
ExaminePixel:
	; get the color (two of the three values) for an index into table array
	movzx	edx, WORD PTR [esi]
	
	; see if we have already allocated a sub-array for the last byte of the color
	cmp	DWORD PTR [eax+edx*4], 0
	lea	edi, [eax+edx*4]
	jne	SHORT NoSubAlloc

	; we haven't yet, so let's do so now
	push	256
	push	8
	push	ebx
;	call	dword ptr __imp__HeapAlloc@12
	call	HeapAlloc
	mov	[edi], eax
	mov	eax, [esp+16]

NoSubAlloc:
	; get the index into the sub-color array (0 - 256) and store
	; a '1' value to indicate this color has been used.  Also loop back
	; and do more if we haven't reached the end of the image pixels
	movzx	ecx, BYTE PTR [esi+2]
	mov	edx, [edi]
	add	esi, 4
	cmp	esi, ebp
	mov	BYTE PTR [ecx+edx], 1
	jb	SHORT ExaminePixel

	; now we are going to loop through the main table array and count
	; all the unique color counts.
	xor	esi, esi
	mov	edi, eax
	lea	ebp, [eax+262144]

align 4
AddCounts:
	; check and see if any sub-arrays have been allocated for this color type
	mov	ecx, [edi]
	test	ecx, ecx
	je	SHORT NotAllocated

	mov	eax, 255

align 4
@@:
	; count all the colors in the 256-byte array
	movzx	edx, byte ptr [ecx+eax]
	add	esi, edx
	sub	eax, 1
	jns	short @B

	; free the 256-byte array now
	push	ecx
	push	0
	push	ebx
;	call	dword ptr __imp__HeapFree@12
	call	HeapFree

NotAllocated:
	; see if we have reached the end of the main table (256 x 256)
	add	edi, 4
	cmp	edi, ebp
	jb	SHORT AddCounts

	; free the main table allocation
	push	[esp+16]
	push	0
	push	ebx
;	call	dword ptr __imp__HeapFree@12
	call	HeapFree

	; restore registers and return the code of unique colors
	mov	eax, esi
	pop	edi
	pop	esi
	pop	ebp
	pop	ebx
	pop	ecx

	ret	12
_CountColors@12 ENDP

END
