TITLE "Draws a image on the DC using a HBITMAP handle"

.586
.model flat
option casemap :none      ; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
include Windows.inc
include Gdi32.inc


; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
include asm_Graphics.inc



.code

; ----------------------------------------------------------------------------
; prototype:
;		void __stdcall DrawBitmapFromRegion(const HDC hdc, const HBITMAP hBitmap, const HRGN hRegion, const LPRECT pRect);
;
; Description:
;		Draws the specified bitmap on to the DC object with the correct size and
;		clipped by region handle
;
; Return value:
;		(none)
; ----------------------------------------------------------------------------

; ---------------------------------------------------------------------------
; variable offests into ESP upon function entering and adjustment of the
; stack for local variables and register preservation
; ===========================================================================
; ESP + 0  = saved EBX value
;
;
; ESP + 4 = (return stack)
;
; ESP + 8 = HDC parameter
; ESP + 12 = HBITMAP parameter
; ESP + 16 = HRGN parameter
; ESP + 20 = LPRECT parameter
align 16
_DrawBitmapFromRegion@16 proc
	; save critical registers
	push ebx
	
	; create an empty region for combining
	xor eax, eax
	push eax
	push eax
	push eax
	push eax
	call CreateRectRgn
	mov ebx, eax
	
	; combine newly created empty region with the passed in region info.
	push RGN_COPY
	push NULL
	push [esp + 16 + 4 + 4]	; HRGN parameter
	push ebx
	call CombineRgn
	
	; Adjust the region offset to start at our x, y location
	mov eax, [esp + 20]	; LPRECT param
	push [eax].RECTL.top
	push [eax].RECTL.left
	push ebx
	call OffsetRgn	
	
	; select the new region into the DC object
	push ebx
	push [esp + 8 + 4]	; HDC param
	call SelectClipRgn
	
	; Draw the bitmap onto the DC object (with clipping)
	mov eax, [esp + 20]	; LPRECT param
	mov ecx, [esp + 12]	; HBITMAP param
	mov edx, [esp + 8]	; HDC param
	
	push eax
	push ecx
	push edx
	call DrawBitmap
	
	; set clipping region back to NULL for this DC
	push NULL
	push [esp + 8 + 4]	; HDC param
	call SelectClipRgn
	
	; release the HRGN handle we created
	push ebx
	call DeleteObject  	

	; restore critical registers	
	pop ebx
	
	ret 16
_DrawBitmapFromRegion@16 endp

END
