TITLE "Draws a image on the DC using a HBITMAP handle"

.586
.model flat
option casemap :none      ; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
include Windows.inc
include Gdi32.inc


; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
include asm_Graphics.inc



.code
; ----------------------------------------------------------------------------
; prototype:
;		void __stdcall DrawTransparentBitmap(const HDC hdc, const HBITMAP hBitmap, const LPRECT pRect, const LPCOLORREF, const BOOL bUsePixel);
;
; Description:
;
;
; Return value:
;		(none)
; ----------------------------------------------------------------------------

; ---------------------------------------------------------------------------
; variable offests into ESP upon function entering and adjustment of the
; stack for local variables and register preservation
; ===========================================================================
; ESP + 0 = saved EBX value
; ESP + 4 = saved ESI value
; ESP + 8 = saved EDI value 
;
;
; ESP + 12 = (return stack)
;
; ESP + 16 = HDC parameter
; ESP + 20 = HBITMAP parameter
; ESP + 24 = LPRECT parameter
; ESP + 28 = LPCOLORREF parameter
; ESP + 32 = BOOL parameter
align 16
_DrawTransparentBitmap@20 proc
	; save critical registers
	push ebx
	push esi
	push edi
	
	; get params into some registers
	mov ecx, [esp + 32]
	mov eax, [esp + 28]
	mov edi, [esp + 24]
	mov esi, [esp + 20]
	
	; create our region data from the HBITMAP now
	push ecx
	push eax
	push [edi].RECTL.bottom
	push [edi].RECTL.right
	push esi
	call CreateRegionFromHBITMAP
	mov ebx, eax
	
	; draw the image using the specified clipping region
	mov eax, [esp + 16]
	push edi
	push ebx
	push esi
	push eax
	call DrawBitmapFromRegion
	
	; release resource on the HRGN data
	push ebx
	call DeleteObject
	
	; restore critical registers
	pop edi
	pop esi
	pop ebx
	
	ret 20
_DrawTransparentBitmap@20 endp

COMMENT #
; -----------------------------------------------------------------------------------------
; Prototype		:  void __stdcall DrawTransparentBitmap(HDC hDC, HBITMAP hBitmap, LPRECT Rect, LPCOLORREF lpColor, BOOL bUsePixel);
; Description	: 
; -----------------------------------------------------------------------------------------
align 16
DrawTransparentBitmap proc public uses ebx esi hDC:HDC, hBitmap:HBITMAP, Rect:dword, TransColor:dword, UsePixel:dword 

	mov	eax, [TransColor]
	mov	ecx, [UsePixel]
	mov	edi, [Rect]
	mov	esi, [hBitmap]
	
	invoke CreateRegionFromHBITMAP, esi, [edi].RECTL.right, [edi].RECTL.bottom, eax, ecx
	mov	ebx, eax
	
	invoke DrawBitmapFromRegion, [hDC], esi, ebx, edi
	
	invoke DeleteObject, ebx
	
	ret 	
DrawTransparentBitmap endp
#

END
