TITLE	FixSingleDot.asm

.486
.model flat      			; memory model
option casemap :none    ; case sensitive


.code
align 16

; ----------------------------------------------------------------------------
; prototype:
; int __stdcall FixSingleDotA(const char* inStr, char* outStr);
; ----------------------------------------------------------------------------

; ---------------------------------------------------------------------------
; variable offests into ESP upon function entering and adjustment of the
; stack for local variables and register preservation
; ===========================================================================
; ESP + 12 = inString parameter
; ESP + 16 = outString parameter

_FixSingleDotA@8 proc near
	; save our critical registers
	push edi
	push esi

	; set up the registers
	mov edi, [esp + 16]
	mov esi, [esp + 12]

	xor eax, eax		; new outBuffer length
	xor ecx, ecx		; walk index into inBuffer
	
	; EDX will be our working register to read and store the bytes

	; we are going to do the first character outside the loop to increase
	; the speed of the loop by not having a extra check condition to make
	; sure we aren't going out of bounds by checking inBuffer[-1]. This
	; will also NULL terminate our outgoing string if the incoming length
	; is zero.
	mov dl, byte ptr [esi]
	cmp dl, 2eh		; '.'
	mov byte ptr [edi], dl
	jne LookForDot

	; we did find it so we need to add an extra '.' to outBuffer
	add eax, 1
	mov byte ptr [edi+eax], 2eh		; '.'
	
	; loop through each character and place into the outBuffer while
	; checking to see if there is a '.' at the beginning of the line
align 4	
LookForDot:
	; check to see if we have reached the end of the buffer
	cmp dl, 0
	je Done
	
	; get the next character we are going to use
	mov dl, byte ptr [esi+ecx+1]

	; update our index and length
	add ecx, 1
	add eax, 1
	
	; write out the character
	mov byte ptr [edi+eax], dl
	
	; see if we found the dot or EOL before the dot.
	cmp dl, 2eh		; '.'
	jne LookForDot
	
	; now look at the previous byte and see if that happens to be the EOL marker
	cmp byte ptr [esi+ecx-1], 0ah		; '\n'
	jne LookForDot
		
	; found a '.' on the first of the line, need to add an additional one
	mov byte ptr [edi+eax+1], 2eh		; '.'
	add eax, 1
	jmp LookForDot	

Done:
	; restore registers
	pop esi
	pop edi
	
	ret 8
_FixSingleDotA@8 endp

END
