TITLE	QuotableEncode.asm

.486
.model flat      			; memory model
option casemap :none    ; case sensitive


.code
align 16

; ----------------------------------------------------------------------------
; prototype:
; int __stdcall QuotedPrintableEncodeA(const char* inStr, char* outStr);
; ----------------------------------------------------------------------------

; ---------------------------------------------------------------------------
; variable offests into ESP upon function entering and adjustment of the
; stack for local variables and register preservation
; ===========================================================================
; ESP + 20 = InString parameter
; ESP + 24 = OutString parameter

_QuotedPrintableEncodeA@8 proc near
	; save our critical registers
	push	ebx
	push	esi
	push	edi
	push	ebp

	; the following registers will contain:
	; eax = length of outgoing string
	; esi = current offset into the incoming string
	; edi = outgoing string
	; ebp = lineoffset variable -- when this value is at 75, we need to wrap the line

	; ebx, ecx and edx are general purpose registers to work with
	
	; set up the registers for the processing
	mov	edi, [esp + 24]	; outString parameter
	mov	esi, [esp + 20]	; inString parameter
	xor	eax, eax
	xor	ebp, ebp

	; keep processing characters and place the processed in the out buffer until we
	; reach a NULL in the incoming buffer
align 4
ProcessChars:
	mov	bl, byte ptr [esi]
	add	esi, 1	
	test	bl, bl
	je	Done
	
	; just add these characters directly to the out going buffer.
	; (bl >= 32 && bl <= 60) || (bl >= 62 && bl <= 126) ||
	; (bl == '\r') || (bl == '\t'))
	
	; check for \n, \r or \t
	cmp	bl, 20h
	jl	CheckLowCharacters
	
	; check for a single dot.
	cmp	bl, 2eh	; '.'
	je EscapeDot

	; check for the '=' symbol
	cmp	bl, 3dh
	je EscapeCharacter
	
	; if (bl >= 32 && bl <= 60), (bl >= 62 && bl <= 126), just directly
	; add this character to the out going buffer, otherwise, we need to
	; turn the ASCII value into hex and prepend a '='.
	cmp	bl, 7eh
	jg	EscapeCharacter

align 4
AddCharacter:
	; save this character in the out going buffer
	mov	byte ptr [edi+eax], bl
	
	; advance all pointers
	add	ebp, 1
	add	eax, 1
	
	; see if we need to wrap the line
	cmp	ebp, 4ch
	jl ProcessChars	

	; add the following three ASCII characters
	mov	byte ptr [edi+eax], 3dh		; '='
	mov	byte ptr [edi+eax+1], 0dh		; '\r'
	mov	byte ptr [edi+eax+2], 0ah		; '\n'
	
	; advance all pointers
	add	eax, 3
	xor	ebp, ebp			; reset our line length variable back to zero
	jmp ProcessChars


align 4
; this section will add only the following three characters to the output
; buffer ('\n', '\r', '\t'). All other characters below a ASCII value of
; 32 will just ignored
CheckLowCharacters:
	cmp	bl, 09h	; '\t'
	je AddCharacter
	
	cmp	bl, 0dh	; '\r'
	je AddCharacter
	
	cmp	bl, 0ah	; '\n'
	jne ProcessChars

	; advance pointers and reset the line length back to zero
	mov	byte ptr [edi+eax], bl
	xor	ebp, ebp
	add	eax, 1
	jmp ProcessChars


align 4
EscapeDot:
	; see if there is a dot on the first character of a line. If so, we need to
	; escape it because SMTP and NNTP protocols might think that this is the end
	; of message marker and ignore the period character by stripping it.
	test	ebp, ebp
	jne AddCharacter

	; escape the dot.
	mov	byte ptr [edi+eax], bl
	mov	byte ptr [edi+eax+1], bl
	
	; advance pointers
	add	ebp, 2
	add	eax, 2
	jmp ProcessChars


align 4
EscapeCharacter:
	; non-valid characters allowed in QuotablePrintable encoding. We need to
	; turn them into Hex values and prepend a '=' symbol before the hex value
	mov	cl, bl
	mov	dl, bl
	and	cl, 0f0h
	and	dl, 00fh
	shr	cl, 4
	add	cl, 30h
	add	dl, 30h

	mov	byte ptr [edi+eax], 61	; '='
	cmp	cl, 3ah
	jl	@F
	add	cl, 7
	
@@:
	mov	byte ptr [edi+eax+1], cl
	cmp	dl, 3ah
	jl @F
	add	dl, 7
	
@@:
	mov	byte ptr [edi+eax+2], dl
	
	; advance our pointers
	add	eax, 3
	add	ebp, 3
	jmp ProcessChars

		
align 4
Done:
	; terminate the final string
	mov byte ptr [edi+eax], 0
	
	; restore our registers
	pop	ebp
	pop	edi
	pop	esi
	pop	ebx
	
	ret 8
_QuotedPrintableEncodeA@8 endp

END	
