TITLE ""

.586
.model flat, stdcall
option casemap :none      ; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
include Windows.inc
include User32.inc


; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
include asm_Memory.inc
include Menu.inc


.code

; ----------------------------------------------------------------------------
; prototype:
;   void DeleteMenuItemDataA(HMENU hMenu);
;
; Description (ANSI/UNICODE version):
;   Deletes all menu items associated with the menu handle. Will also delete
;   any sub-menu items too. This function is recursive.
;
; Returns:
;   (none)
; ============================================================================
align 16
DeleteMenuItemData proc public uses esi edi hMenu:dword
    LOCAL mmi : MENUITEMINFO
     
    ; if we have a NULL menu handle return
    mov eax, [hMenu]
    test eax, eax
    je short RecursiveEnd
    
    ; get the count of menu items for this menu
    push eax
    call GetMenuItemCount
    
    ; are there no items or function fails, return
    test eax, eax
    jle short RecursiveEnd
    mov edi, eax
    
    ; set up pre-loop values
    lea esi, [mmi]
    sub edi, 1
    
    ; set up the 'mmi' information which we'll need to get the menu item's data
    mov [mmi.cbSize], sizeof MENUITEMINFO
    mov [mmi.fMask], MIIM_DATA

align 4
DeleteItem:
    ; get the menu item information
    mov eax, [hMenu]
    push esi
    push 1
    push edi
    push eax
    call GetMenuItemInfo
    
    ; if the item's data has been allocated, free it
    mov eax, [mmi.dwItemData]
    test eax, eax
    je short ContinueDeleteItem
    
    ; free memory of item's data
    push eax
    call fmFree

align 4    
ContinueDeleteItem:
    ; if there are sub-menus on this menu, process them now
    push edi
    push [hMenu]
    call GetSubMenu
    
    push eax
    call DeleteMenuItemData ; recursive to process this submenu
    
    sub edi, 1
    jns short DeleteItem 

align 4
RecursiveEnd:
    ; return back to caller
    ret
DeleteMenuItemData endp

END
