TITLE ""

.586
.model flat, stdcall			; memory model
option casemap :none			; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
include Windows.inc
include User32.inc


; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
include asm_Memory.inc
include ..\menu.inc

.code

; ----------------------------------------------------------------------------
; prototype:
;   void CreateMenuDataW(IN HMENU hMenu, IN BOOL bMainMenu, IN HFONT hFont);
;
; Description: (UNICODE version):
;   Creates all menu items associated with the menu handle. Will also create
;   any sub-menu items too. This function is recursive.
;
; Returns:
;   (none)
; ============================================================================
align 16
CreateMenuDataW proc public uses ebx esi edi hMenu:HMENU, bMainMenu:DWORD, hFont:HFONT
	LOCAL mmi				: MENUITEMINFO
	LOCAL wszBuffer[256]	: word
	
	; test for a value menu handle
	mov	eax, [hMenu]
	test eax, eax
	je Finished
	
	; get the count of menu items in this level
	push eax
	call GetMenuItemCount
	
	; test for a non-zero value
	test eax, eax
	jle Finished
	mov	ebx, eax
	
	; zero out our mmi structure
	xor	eax, eax
	mov	ecx, sizeof MENUITEMINFO / 4		; currently divisible by four
	lea	edi, [mmi]
	rep	stosd

	; set the mmi structure size and make our loop count zero based
	mov	[mmi.cbSize], sizeof MENUITEMINFO
	sub	ebx, 1
	
	; set the address of our string buffer to ESI for constant use
	lea	esi, [wszBuffer]
	
align 4
ProcessItems:
	; save the item text in the item structure
	lea	edi, [mmi]
	mov	[mmi.fMask], MIIM_TYPE
	mov	[mmi.dwTypeData], esi
	mov	[mmi.cch], 0ffh
	push edi
	push TRUE
	push ebx
	push [hMenu]
	call GetMenuItemInfoW

	; allocate a item structure and enough room for the text
	mov	edx, [mmi.cch]
	shl edx, 1
	add	edx, sizeof MenuItem
	add edx, 2
	push edx
	call fmAlloc
			
	; fill in the user-data for this menu item now
	mov	edx, [hFont]
	mov	ecx, [mmi.cch]
	mov	[eax].MenuItem.hFont, edx
	mov	[eax].MenuItem.cchItemText, ecx
	
	mov	edx, [bMainMenu]
	test ecx, ecx
	mov	[eax].MenuItem.bMainMenu, edx
	je short @F
	
	; we have a non-seperator (menu item has text), so copy it.
	shl ecx, 1
	lea	edi, [eax].MenuItem.szItemText
	rep	movsb
	
	; need to restore the ESI since we lost it from the copy
	lea	esi, [wszBuffer]

@@:
	; change the item to an owner-drawn item and save the address of
	; the item structure as item data
	lea edx, [mmi] 
	mov	[mmi.dwItemData], eax
	or [mmi.fType], MFT_OWNERDRAW
	mov	[mmi.fMask], MIIM_TYPE or MIIM_DATA
	push edx
	push TRUE
	push ebx
	push [hMenu]
	call SetMenuItemInfoW

	; now re-cursively go through all the sub-menus of this menu item.
	push ebx
	push [hMenu]
	call GetSubMenu
	
	push [hFont]
	push FALSE
	push eax
	call CreateMenuDataW

	sub	ebx, 1
	jns	ProcessItems
			
Finished:
	ret
CreateMenuDataW endp

END
