TITLE "Adds a new element to the array"

.586
.model flat			; memory model
option casemap:none	; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
; (none)

; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
include asm_Memory.inc
include asm_faArray.inc


.code

; ----------------------------------------------------------------------------
; prototype:
; LONG __stdcall faAddElement(LPVOID* pArray, LPCVOID pElement);
; ----------------------------------------------------------------------------
align 16
_faAddElement@8 proc public
	; save the ESI register
	push	esi
	mov	esi, [esp+8]
	
	; check our params
	xor	eax, eax
	push	eax
	push	eax
	push	esi
	call  faCheckParameters
	
	; check result
	test	eax, eax
	je		@F
	
	; return result since it wasn't success
	pop	esi
	ret 8
	
@@:
	; check and see if we are at the end of the allocation
	; and we need to add more element room
	mov	eax, [esi].ArrayObject.AllocSize
	mov	edx, [esi].ArrayObject.Count
	cmp	edx, eax
	jne	AddElemToArray
	
	; we are at the end here, so lets increase the element
	; allocation room
	add	eax, AO_ALLOC_SIZE
	shl	eax, 2	
	push eax
	push [esi].ArrayObject.Array
	call fmResize
	
	; make sure the new allocation succeeded
	test	eax, eax
	jne	@F
	
	; failed, so return out of memory error status
	mov	eax, AO_OUT_OF_MEMORY
	pop	esi
	ret 8

@@:
	; allocation succeeded at this point, lets reset the memory pointer
	; to the new allocation and adjust the AllocSize
	add	[esi].ArrayObject.AllocSize, AO_ALLOC_SIZE
	mov	[esi].ArrayObject.Array, eax

	; set up the edx register with the current array element count	
	mov	edx, [esi].ArrayObject.Count

AddElemToArray:
	; add the element to the array at the 'Count' index
	mov	eax, [esp+12]
	mov	ecx, [esi].ArrayObject.Array
	mov	[ecx+edx*4], eax
	
	; return the index where we added the element
	add	dword ptr [esi].ArrayObject.Count, 1
	mov	eax, edx
	
	pop	esi
	ret 8
_faAddElement@8 ENDP

END
