TITLE "Removes an element from the array by index value"

.586
.model flat			; memory model
option casemap:none	; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
; (none)

; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
include asm_faArray.inc


.code

; ----------------------------------------------------------------------------
; prototype:
; LONG __stdcall faRemoveElementByIndex(ArrayObject* Object, const LONG Index);
; ----------------------------------------------------------------------------
align 16
_faRemoveElementByIndex@8 proc public
	; save the ESI register
	push	esi
	mov	esi, [esp+8]
	
	; check our params
	push	1
	push	[esp+16]
	push	esi
	call  faCheckParameters
	
	; check result status
	test	eax, eax
	jne	Done	

	; get our index parameter and make it one based
	mov	eax, [esp+12]
	add	eax, 1
	
	; check to see if it is the last element in array
	cmp	eax, [esi].ArrayObject.Count
	mov	edx, [esi].ArrayObject.Array
	je	NoBlockMove

	; need to move the array elements up by one and NULL out the last index
	; to do this, the follow registers will be used:
	; EDI = destination address in array (element address)
	; ESI = source address in array (element address)
	; ECX = number of bytes to move (always divisible by four)
	mov	ecx, [esi].ArrayObject.Count
	sub	ecx, eax

	; move array pointers up by one element
	push	edi	
	lea	edi, [edx + eax * 4 - 4]	; need to zero base the index value
	lea	esi, [edx + eax * 4] 		; keep the index value one based
	rep 	movsd

	; now that the array has been shifted down by one element, we need to zero
	; out the last elements pointer so it doesn't point to anything
	mov	ecx, [esp+12]	; object param
	xor	eax, eax
	mov	[edi], eax
	sub	dword ptr [ecx].ArrayObject.Count, 1

	pop	edi
	pop	esi
	ret	8
	
NoBlockMove:
	; set the last element pointer to NULL
	; EAX contains array count (one based)
	; EDX contains array address
	lea	ecx, [edx + eax * 4 - 4]
	sub	dword ptr [esi].ArrayObject.Count, 1
	xor	eax, eax	
	mov	[ecx], eax
	
Done:
	pop	esi
	ret	8
_faRemoveElementByIndex@8 ENDP

END
