TITLE "Removes an element based on pointer comparison"

.586
.model flat			; memory model
option casemap:none	; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
; (none)

; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
include asm_faArray.inc


.code

; ----------------------------------------------------------------------------
; prototype:
; LONG __stdcall faRemoveElementByPointer(ArrayObject* Object, LPCVOID pElement);
; ----------------------------------------------------------------------------
align 16
_faRemoveElementByPointer@8 proc public
	; save critical registers
	push	ebp
	push	edi
	push	esi
	push	ebx

	; copy parameter variables into registers
	mov	esi, [esp + 20]	; ArrayObject* Object
	mov	ebx, [esp + 24]	; LPCVOID pElement
	mov	ebp, esi
		
	; find our element in the array
	push	ebx
	push	esi
	call	faGetIndexByElement
	
	; make sure we found it
	cmp	eax, AO_SUCCESS
	jl		Done
	
	; set our index value one based
	add	eax, 1
	
	; check to see if it is the last element in array
	cmp	eax, [esi].ArrayObject.Count
	mov	edx, [esi].ArrayObject.Array
	je	NoBlockMove

	; need to move the array elements up by one and NULL out the last index
	; to do this, the follow registers will be used:
	; EDI = destination address in array (element address)
	; ESI = source address in array (element address)
	; ECX = number of bytes to move (always divisible by four)
	mov	ecx, [esi].ArrayObject.Count
	sub	ecx, eax

	; move array pointers up by one element
	lea	edi, [edx + eax * 4 - 4]	; need to zero base the index value
	lea	esi, [edx + eax * 4] 		; keep the index value one based
	rep 	movsd

	; now that the array has been shifted down by one element, we need to zero
	; out the last elements pointer so it doesn't point to anything
	xor	eax, eax
	mov	[edi], eax
	sub	dword ptr [ebp].ArrayObject.Count, 1
	jmp Done

	
NoBlockMove:
	; set the last element pointer to NULL
	; EAX contains array count (one based)
	; EDX contains array address
	lea	ecx, [edx + eax * 4 - 4]
	sub	dword ptr [ebp].ArrayObject.Count, 1
	xor	eax, eax	
	mov	[ecx], eax
	
Done:
	pop	ebx
	pop	esi
	pop	edi
	pop	ebp
	
	ret 8
_faRemoveElementByPointer@8 ENDP

END
