TITLE "Create a displayable date and/or time string in ANSI characters"

.486
.model flat
option casemap :none      ; case sensitive

include Kernel32.inc

.const
DATE_FORMAT_STRING db 'M',0,'M',0,'/',0,'d',0,'d',0,'/',0,'y',0,'y',0,'y',0,'y', 0,0h
TIME_FORMAT_STRING db 'h',0,'h',0,':',0,'m',0,'m',0,':',0,'s',0,'s',0,' ',0,'t',0,'t', 0,0h

.code
align 16
; ----------------------------------------------------------------------------
; prototype:
;   void GetDateTimeStringW(SYSTEMTIME* pSysTime, wchar_t* dtString, wchar_t* tmString);
;
; description:
;	Will either get the current local time or use the passed in time (depending
;	if pSysTime parameter is NULL or not and create a displayable date and time
;	strings.
;
; ESP + 24 = pSysTime parameter 
; ESP + 28 = dtString parameter
; ESP + 32 = tmString parameter
;
; ESP +  4 = SYSTEMTIME stack variable
; ============================================================================
_GetDateTimeStringW@12 proc near
	; make room on the stack for a SYSTEMTIME structure in case we need to
	; get the local time.
	sub esp, 16
	
	; we are going to use ebx to keep the address of the SYSTEMTIME
	; stack variable so we need to preserve this register value for windows.
	push ebx
	
	; make sure our date and time strings aren't a NULL pointer
	mov eax, [esp + 28]
	mov ecx, [esp + 32]
	test eax, eax
	je @F
	test ecx, ecx
	je @F
	
	; now we need to check and see if the first parameter is a NULL pointer or
	; pointing to the SYSTEMTIME information they would like to use. If it is
	; NULL, we'll call GetLocalTime to get the current date/time.
	mov ebx, [esp + 24]
	test ebx, ebx
	jne DoFormatting
	
	; it is a NULL pointer so let's get the current date/time into a
	; SYSTEMTIME structure to work with
	lea ebx, [esp + 4]
	push ebx
	call GetLocalTime
	
DoFormatting:
	; format the date string into parameter
	push 0fh	; max string length
	push eax
	push offset flat:DATE_FORMAT_STRING
	push ebx
	push 40000000H	; LOCALE_USE_CP_ACP
	push 400H		; LOCALE_USER_DEFAULT
	call GetDateFormatW
	
	; format the time string into parameter
	push 0fh	; max string length
	push eax
	push offset flat:TIME_FORMAT_STRING
	push ebx
	push 40000000H	; LOCALE_USE_CP_ACP
	push 400H		; LOCALE_USER_DEFAULT
	call GetTimeFormatW

@@:
	; restore our register
	pop ebx

	add esp, 16	
	ret 12	
_GetDateTimeStringW@12 endp

END
