; ****************************************************************************
;   This file contains all defines (equates), structures and ANSI/UNICODE
;   function prototypes for the WinOS Kernel32.DLL. ANSI or UNICODE specific
;   function prototypes are defined in their respective .INC file.
;
;   You may use or modify this code freely without any restrictions.
;
;   v1.0 - Initial coding of the file   Oct 10, 2006  4:00 PM
; ****************************************************************************

; ----------------------------------------------------------------------------
; WinOS Kernel32.DLL library specific defines and global variables
; ============================================================================
MEM_COMMIT			equ 1000h
MEM_RESERVE			equ 2000h
MEM_DECOMMIT		equ 4000h
MEM_RELEASE			equ 8000h

INVALID_HANDLE_VALUE					equ 0ffffffffh
INVALID_FILE_SIZE						equ 0ffffffffh
INVALID_SET_FILE_POINTER			equ 0ffffffffh
INVALID_FILE_ATTRIBUTES				equ 0ffffffffh

FILE_BEGIN								equ 0h
FILE_CURRENT							equ 1h
FILE_END									equ 2h

GENERIC_ALL								equ 10000000h
GENERIC_EXECUTE						equ 20000000h
GENERIC_WRITE							equ 40000000h
GENERIC_READ							equ 80000000h

FILE_SHARE_READ						equ 1h
FILE_SHARE_WRITE						equ 2h
FILE_SHARE_DELETE						equ 4h

FILE_ATTRIBUTE_READONLY					equ   1h
FILE_ATTRIBUTE_HIDDEN					equ   2h
FILE_ATTRIBUTE_SYSTEM					equ   4h
FILE_ATTRIBUTE_DIRECTORY				equ  10h
FILE_ATTRIBUTE_ARCHIVE					equ  20h
FILE_ATTRIBUTE_DEVICE					equ  40h
FILE_ATTRIBUTE_NORMAL					equ  80h
FILE_ATTRIBUTE_TEMPORARY				equ 100h
FILE_ATTRIBUTE_SPARSE_FILE				equ 200h
FILE_ATTRIBUTE_REPARSE_POINT			equ 400h
FILE_ATTRIBUTE_COMPRESSED				equ 800h
FILE_ATTRIBUTE_OFFLINE					equ 1000h
FILE_ATTRIBUTE_NOT_CONTENT_INDEXED	equ 2000h
FILE_ATTRIBUTE_ENCRYPTED				equ 4000h

FILE_NOTIFY_CHANGE_FILE_NAME		equ  01h
FILE_NOTIFY_CHANGE_DIR_NAME		equ  02h
FILE_NOTIFY_CHANGE_ATTRIBUTES		equ  04h
FILE_NOTIFY_CHANGE_SIZE				equ  08h
FILE_NOTIFY_CHANGE_LAST_WRITE		equ  10h
FILE_NOTIFY_CHANGE_LAST_ACCESS	equ  20h
FILE_NOTIFY_CHANGE_CREATION		equ  40h
FILE_NOTIFY_CHANGE_SECURITY		equ 100h

; File creation flags must start at the high end since they
; are combined with the attributes

FILE_FLAG_SEQUENTIAL_SCAN			equ 08000000h
FILE_FLAG_NO_BUFFERING				equ 20000000h
FILE_FLAG_RANDOM_ACCESS				equ 10000000h
FILE_FLAG_WRITE_THROUGH				equ 80000000h

CREATE_NEW								equ 1h
CREATE_ALWAYS							equ 2h
OPEN_EXISTING							equ 3h
OPEN_ALWAYS								equ 4h
TRUNCATE_EXISTING						equ 5h

; Heap Memory Flags (obsolete are not included)
HEAP_NO_SERIALIZE           equ 1h
HEAP_GENERATE_EXCEPTIONS    equ 4h
HEAP_ZERO_MEMORY            equ 8h

; Global Memory Flags (obsolete are not included)
GMEM_FIXED      equ 0h
GMEM_MOVEABLE   equ 2h
GMEM_ZEROINIT   equ 40h
GHND            equ (GMEM_MOVEABLE or GMEM_ZEROINIT)
GPTR            equ (GMEM_FIXED or GMEM_ZEROINIT)

; generic defines
MAX_PATH        equ 260


; ----------------------------------------------------------------------------
; structure definitions --- ANSI/UNICODE non-specific
; ============================================================================
FILETIME STRUCT
    dwLowDateTime   dword ?
    dwHighDateTime  dword ?
FILETIME ENDS

SYSTEMTIME STRUCT
    wYear           word ?
    wMonth          word ?
    wDayOfWeek      word ?
    wDay            word ?
    wHour           word ?
    wMinute         word ?
    wSeconds        word ?
    wMilliseconds   word ?
SYSTEMTIME ENDS

TIME_ZONE_INFORMATION STRUCT
    Bias            dword ?
    StandardName    word 32 dup (?)
    StandardDate    SYSTEMTIME<>
    StandardBias    dword ?
    DaylightName    word 32 dup (?)
    DaylightDate    SYSTEMTIME<>
    DaylightBias    dword ?    
TIME_ZONE_INFORMATION ENDS


; ----------------------------------------------------------------------------
; Function definitions --- ANSI/UNICODE non-specific
; ============================================================================

; Heap memory functions -- Used for small to medium memory allocations (less then 1meg in size)
externdef stdcall GetProcessHeap    :proto stdcall
externdef stdcall GetProcessHeaps   :proto stdcall :dword, :dword
externdef stdcall HeapAlloc         :proto stdcall :dword, :dword, :dword
externdef stdcall HeapCompact       :proto stdcall :dword, :dword
externdef stdcall HeapCreate        :proto stdcall :dword, :dword, :dword
externdef stdcall HeapDestroy       :proto stdcall :dword
externdef stdcall HeapFree          :proto stdcall :dword, :dword, :dword
externdef stdcall HeapReAlloc       :proto stdcall :dword, :dword, :dword, :dword
externdef stdcall HeapSize          :proto stdcall :dword, :dword, :dword

; Windows NT or higher Heap prototypes
externdef stdcall HeapLock          :proto stdcall :dword
externdef stdcall HeapUnlock        :proto stdcall :dword
externdef stdcall HeapValidate      :proto stdcall :dword, :dword, :dword
externdef stdcall HeapWalk          :proto stdcall :dword, :dword

; Windows XP or higher Heap prototypes
externdef stdcall HeapQueryInformation  :proto stdcall :dword, :dword, :dword, :dword, :dword
externdef stdcall HeapSetInformation    :proto stdcall :dword, :dword, :dword, :dword

; Virtual memory functions -- Used for large memory allocations (greater then 1meg+ in size)
externdef stdcall VirtualAlloc		:proto stdcall :dword, :dword, :dword, :dword
externdef stdcall VirtualFree			:proto stdcall :dword, :dword, :dword

; Global memory functions -- Slower then the Heap memory functions
externdef stdcall GlobalAlloc       :proto stdcall :dword, :dword
externdef stdcall GlobalDiscard     :proto stdcall :dword
externdef stdcall GlobalFree        :proto stdcall :dword
externdef stdcall GlobalHandle      :proto stdcall :dword
externdef stdcall GlobalLock        :proto stdcall :dword
externdef stdcall GlobalReAlloc     :proto stdcall :dword, :dword, :dword
externdef stdcall GlobalSize        :proto stdcall :dword
externdef stdcall GlobalUnlock      :proto stdcall :dword
; externdef stdcall GlobalFlags      :proto stdcall :dword      (16-bit windows only)

; File Management Functions
externdef stdcall CloseHandle       :proto stdcall :dword
externdef stdcall FindClose			:proto stdcall :dword
externdef stdcall FindNextFile		:proto stdcall :dword, :dword
externdef stdcall GetFileSize       :proto stdcall :dword, :dword
externdef stdcall ReadFile          :proto stdcall :dword, :dword, :dword, :dword, :dword
externdef stdcall ReadFileEx        :proto stdcall :dword, :dword, :dword, :dword, :dword
externdef stdcall SetFilePointer    :proto stdcall :dword, :dword, :dword, :dword
externdef stdcall SetFilePointerEx  :proto stdcall :dword, :dword, :dword, :dword, :dword
externdef stdcall WriteFile         :proto stdcall :dword, :dword, :dword, :dword, :dword
externdef stdcall WriteFileEx       :proto stdcall :dword, :dword, :dword, :dword, :dword

; Time Management Functions
externdef stdcall GetLocalTime		:proto stdcall :dword

; The following functions are used with system time.
externdef stdcall GetSystemTime             :proto stdcall :dword 
externdef stdcall GetSystemTimeAdjustment   :proto stdcall :dword, :dword, :dword 
externdef stdcall SetSystemTime             :proto stdcall :dword 
externdef stdcall SetSystemTimeAdjustment   :proto stdcall :dword 
externdef stdcall SystemTimeToFileTime       :proto stdcall :dword, :dword 
externdef stdcall SystemTimeToTzSpecificLocalTime   :proto stdcall :dword, :dword, :dword 
externdef stdcall TzSpecificLocalTimeToSystemTime   :proto stdcall :dword, :dword, :dword 

; The following functions are used with local time.
externdef stdcall FileTimeToLocalFileTime   :proto stdcall :dword, :dword 
externdef stdcall GetLocalTime              :proto stdcall :dword 
externdef stdcall GetTimeZoneInformation    :proto stdcall :dword 
externdef stdcall SetLocalTime              :proto stdcall :dword 
externdef stdcall SetTimeZoneInformation    :proto stdcall :dword 

; The following functions are used with file time.
externdef stdcall CompareFileTime           :proto stdcall :dword, :dword 
externdef stdcall FileTimeToLocalFileTime   :proto stdcall :dword, :dword 
externdef stdcall FileTimeToSystemTime      :proto stdcall :dword, :dword 
externdef stdcall GetFileTime               :proto stdcall :dword, :dword, :dword, :dword 
externdef stdcall GetSystemTimeAsFileTime   :proto stdcall :dword 
externdef stdcall LocalFileTimeToFileTime   :proto stdcall :dword, :dword 
externdef stdcall SetFileTime               :proto stdcall :dword, :dword, :dword, :dword 
externdef stdcall SystemTimeToFileTime      :proto stdcall :dword, :dword 

; The following functions are used with MS-DOS date and time.
externdef stdcall DosDateTimeToFileTime     :proto stdcall :word, :word, :dword 
externdef stdcall FileTimeToDosDateTime     :proto stdcall :dword, :dword, :dword 

; The following functions are used with Windows time.
externdef stdcall GetTickCount              :proto stdcall 
externdef stdcall GetSystemTimes            :proto stdcall :dword, :dword, :dword 

; The following functions are used with high-resolution timers.
externdef stdcall QueryPerformanceCounter   :proto stdcall :dword 
externdef stdcall QueryPerformanceFrequency :proto stdcall :dword 


; ----------------------------------------------------------------------------
; Function definitions used in the WinOS User32.DLL --- ANSI or UNICODE based
; on define during assembling
; ============================================================================
IF _UNICODE eq 1	; same definition as the MFC library (windows API uses UNICODE)
	include unicode\Kernel32.inc
ELSE
	include ansi\Kernel32.inc
ENDIF
