// ---------------------------------------------------------------------------
// Graphics.h
// ===========================================================================
#if !defined(AFX_ASM_GRAPHICS_H__A9DB83DB_A9FD_11D0_BFD1_444553540000__INCLUDED_)
#define AFX_ASM_GRAPHICS_H__A9DB83DB_A9FD_11D0_BFD1_444553540000__INCLUDED_


#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#if defined(__cplusplus)
extern "C" {
#endif


// ----------------------------------------------------------------------------
// structures used
// ----------------------------------------------------------------------------
typedef struct BMBitsData
{
	HBITMAP	hBitmap;
	POINT		ptDimensions;
	HBITMAP	hDibSection;
	LPVOID	pBits;
} BMBitsData;

typedef struct RenderObject
{
	HBITMAP	hBitmap;
	HRGN		hRegion;
} typeRenderObject;



// ---------------------------------------------------------------------------
// Description:
//		Gets the bits associated to the HBITMAP handle in the pBits variable.
//		Callee will need to free the pBits variable when done to avoid a memory
//		leak by calling DeleteObject(BitmapBitsInfo->hDibSection)
// ---------------------------------------------------------------------------
void __stdcall GetBitmapData(
	IN OUT BMBitsData* pBitmapInfo);


// ---------------------------------------------------------------------------
// Description:
//		Creates a HRGN from the Bitmap data excluding the specified color.
//
//	Returns:
//		NULL on failure
//		HRGN handle of the region. 
// ---------------------------------------------------------------------------
HRGN __stdcall CreateRegion(
	IN LPCVOID pData,
	IN const int nWidth,
	IN const int nHeight,
	IN const COLORREF crColor);


// ---------------------------------------------------------------------------
// Description:
//		Creates a HRGN from a HBITMAP handle excluding the specified color.
//
//	Returns:
//		NULL on failure
//		HRGN handle of the region. 
// ---------------------------------------------------------------------------
HRGN __stdcall CreateRegionFromHBITMAP(
	IN HBITMAP hBitmap,
	IN const int nWidth,
	IN const int nHeight,
	IN const COLORREF crColor,
	IN BOOL bUse1stPixel);


// ---------------------------------------------------------------------------
// Description:
//		Draws the HBITMAP image onto the specified DC object.
// ---------------------------------------------------------------------------
void __stdcall DrawBitmap(
	IN HDC hDC,
	IN HBITMAP hBitmap,
	IN LPRECT pRect);


// ---------------------------------------------------------------------------
// Description:
//		Draws the HBITMAP image onto the specified DC object with x, y offset
//		into the source image
// ---------------------------------------------------------------------------
void __stdcall DrawBitmapFromOffset(
	IN HDC hDC,
	IN HBITMAP hBitmap,
	IN LPRECT pRect,
	IN LPPOINT pOffset);


// ---------------------------------------------------------------------------
// Description:
//		Draws the HBITMAP image onto the specified DC object.  Uses a HRGN
//		to clip the drawing (transparency) of non-interest areas.
// ---------------------------------------------------------------------------
void __stdcall DrawBitmapFromRegion(
	IN const HDC hdc,
	IN const HBITMAP hBitmap,
	IN const HRGN hRegion,
	IN const LPRECT pRect);


// ---------------------------------------------------------------------------
// Description:
//		Draws the HBITMAP image onto the specified DC object.  Generates a
//		HRGN to clip the drawing (transparency) of non-interest areas.
// ---------------------------------------------------------------------------
void __stdcall DrawTransparentBitmap(
	IN HDC hDC,
	IN HBITMAP bmCheck,
	IN LPRECT pRect,
	IN LPCOLORREF lpColor,
	IN BOOL bUsePixel);


// ---------------------------------------------------------------------------
// Description:
//		Converts a HBITMAP object to Grayscale.
// ---------------------------------------------------------------------------
void __stdcall GrayScaleFromHBITMAP(
	IN HBITMAP hBitmap);


// ---------------------------------------------------------------------------
// Description:
//		Creates a HBITMAP object from raw graphic data --- raw graphic data must
//		be a BMP format.  Can optionally creates a HRGN for transparency effect
//		if specified by passing a valid COLORREF pointer.  This routine can also
//		expand or shrink the image when creating it.
//
//	NOTE:
//		pRender will be filled in with the correct HBITMAP and HRGN objects
//		if successfully rendered the image.
// ---------------------------------------------------------------------------
HBITMAP __stdcall bmpRenderImage(
	IN LPVOID pData,
	IN LPSIZE lpSize);


// ---------------------------------------------------------------------------
// Description:
// ---------------------------------------------------------------------------
int __stdcall CountColors(
	IN LPCVOID pImgData,
	IN const int CX,
	IN const int CY);


#if defined(__cplusplus)
}
#endif

// ===========================================================================
#endif // #define AFX_ASM_GRAPHICS_H__A9DB83DB_A9FD_11D0_BFD1_444553540000__INCLUDED_
