// ---------------------------------------------------------------------------
// asm.h
// ===========================================================================
#if !defined(AFX_ASM_ZIP_H__A9DB83DB_A9FD_11D0_BFD1_444553540000__INCLUDED_)
#define AFX_ASM_ZIP_H__A9DB83DB_A9FD_11D0_BFD1_444553540000__INCLUDED_


#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#if defined(__cplusplus)
extern "C" {
#endif


#pragma pack(1)
typedef struct CentralDirectoryBaseHeader
{
   WORD	VerNeeded;		// version needed to extract
   WORD	Flag;				// general purpose bit flag
   WORD	Method;			// compression method
   WORD	ModTime;			// last modified time
   WORD	ModDate;			// last modified date
   DWORD	Crc32;			// CrC32
   DWORD	CompSize;		// compressed size
   DWORD	UnCompSize;		// uncompressed size
   WORD	FileNameSize;	// filename length
   WORD	ExtraField;		// extra field length
} typeCentralDirectoryBaseHeader, *lpCentralDirectoryBaseHeader;	// 26 bytes

typedef struct CentralDirectoryHeader
{
	BYTE		Signature[4];	// central file header signature (0x02014b50)
	WORD		VersionMadeBy;	// version of zip used to create
	CentralDirectoryBaseHeader Base;
	WORD		CommentSize;	// file comment info size
	WORD		DiskStart;		// disk start number
	WORD		InternalAttr;	// internal file attributes
	DWORD		ExternalAttr;	// external file attributes (attribues on disk)
	DWORD		dwOffset;		// relative offset of local header
} typeCentralDirectoryHeader, *lpCentralDirectoryHeader;	// 46 bytes
#pragma pack()

typedef struct ZFIO
{
	LPBYTE	pMemPosition;
	DWORD		dwFileSize;
	LPBYTE	pFileBuffer;
} typeZFIO, *lpZFIO;

typedef struct ZENTRY
{
	CentralDirectoryHeader* pHeader;
	LPTSTR	pFileName;
} typeZENTRY, *lpZENTRY;

typedef struct ZDIRECTORY
{
	long		Count;
	ZENTRY**	Array;
} typeZDIRECTORY, *lpZDIRECTORY;

typedef struct ZARCHIVE
{
	ZFIO*				pIO;
	ZDIRECTORY*		pDirectory;
} typeZARCHIVE, *lpZARCHIVE;

typedef enum eZipError
{
	noError,		// no error
	generic,		// unknown error
	streamEnd,			// zlib library errors
	needDict,			//
	errNo,				//
	streamError,		//
	dataError,			//
	memError,			//	
	bufError,			//
	versionError,		//
	noCentralDir,
	badZipFile,		// damaged or not a zip file
	badCrc,			// crc mismatched
	tooLongFileName,	// the file name of the file added to the archive is too long
	badPassword,	// incorrect password set for the file being decrypted
	cdirNotFound,		///< the central directory was not found in the archive
	dirWithSize,		// during testing: found the directory with the size greater than 0
	zipDirEmpty,		// empty zip file
} enumZipError;

#define SLASH_WINDOWS	TCHAR('\\')
#define SLASH_UNIX		TCHAR('/')



// ANSI and UNICODE ASM prototypes
void __stdcall zaCloseArchive(IN ZARCHIVE* Instance);


// UNICODE ASM prototypes
eZipError __stdcall zaOpenArchiveW(OUT ZARCHIVE* Instance, IN const wchar_t* pwszFileSpec);
wchar_t*  __stdcall zaLoadEntryByIndexW(IN ZARCHIVE* Instance, IN const int Idx, OUT HGLOBAL* BufHandle);
wchar_t*  __stdcall zaLoadEntryByNameW(IN ZARCHIVE* Instance, IN const wchar_t* pwszFileSpec, OUT HGLOBAL* BufHandle);


// ANSI ASM prototypes
eZipError __stdcall zaOpenArchiveA(OUT ZARCHIVE* Instance, IN const char* pszFileSpec);
char*     __stdcall zaLoadEntryByIndexA(IN ZARCHIVE* Instance, IN const int Idx, OUT HGLOBAL* BufHandle);
char*     __stdcall zaLoadEntryByNameA(IN ZARCHIVE* Instance, IN const char* pszFileSpec, OUT HGLOBAL* BufHandle);


// Generic defines to allow a single name to cover both types
#if defined(_UNICODE)
    #define zaOpenArchive       zaOpenArchiveW
    #define zaLoadEntryByIndex  zaLoadEntryByIndexW
    #define zaLoadEntryByName   zaLoadEntryByNameW
#else
    #define zaOpenArchive       zaOpenArchiveA
    #define zaLoadEntryByIndex  zaLoadEntryByIndexA
    #define zaLoadEntryByName   zaLoadEntryByNameA
#endif



#if defined(__cplusplus)
}
#endif

// ===========================================================================
#endif // #define AFX_ASM_ZIP_H__A9DB83DB_A9FD_11D0_BFD1_444553540000__INCLUDED_
