TITLE "Base procedure for calling the Win32 CreateFile API"

.486
.model flat				; memory model
option casemap:none	; case sensitive

include kernel32.inc

.code

align 16
; ----------------------------------------------------------------------------
; prototype:
;   BOOL __stdcall ReadWriteFileInternal(HANDLE hFile, void* Buffer, size_t BufSize);
;
; description:
;
; returns:
;
; ESP +  8 = hFile parameter
; ESP + 12 = Buffer parameter
; ESP + 16 = Length parameter
; ============================================================================
ReadWriteFileInternal PROC NEAR

	; check to make sure our handle value is valid
	mov eax, [esp + 8]
	cmp eax, INVALID_HANDLE_VALUE
	je Failed
	
	; set up registers for our ReadFile() or WriteFile() API call and make
	; the call. EBX will have the address of which function we are going
	; to call.
	lea edx, [esp + 16]
	mov ecx, [esp + 16]
	
	; make the API call now (EBX will have either a ReadFile() or WriteFile())
	push 0
	push edx
	push ecx	

	; get the buffer and set up first two params to API call
	mov edx, [esp + 24]
	push edx
	push eax

	; call ReadFile() or WriteFile API
	call ebx	
	
	; assume the return status of ReadFile/WriteFile is good enough and use it
	; for our return status code
	pop ebx
	ret 12
	
Failed:
	xor eax, eax
	pop ebx
	ret 12
ReadWriteFileInternal ENDP


align 16
; ----------------------------------------------------------------------------
; prototype:
;   BOOL __stdcall fileWrite(HANDLE hFile, const void* Buffer, size_t BufSize);
;
; description:
;
; returns:
; ============================================================================
_fileWrite@12 PROC NEAR

	; set up registers for our WriteFile() api call.
	push ebx
	mov ebx, WriteFile
	
	; jump to the common code now that we have the correct values in our
	jmp ReadWriteFileInternal
	
_fileWrite@12 ENDP


align 16
; ----------------------------------------------------------------------------
; prototype:
;   BOOL __stdcall fileRead(HANDLE hFile, void* Buffer, size_t BufSize);
;
; description:
;
; returns:
; ============================================================================
_fileRead@12 PROC NEAR

	; set up registers for our WriteFile() api call.
	push ebx
	mov ebx, ReadFile
	
	; jump to the common code now that we have the correct values in our
	jmp ReadWriteFileInternal
	
_fileRead@12 ENDP

END
