TITLE "Allocates memory from the process heap"

.586
.model flat             ; memory model
option casemap :none    ; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
include Kernel32.inc


; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
include asm_Memory.inc


.code

; ----------------------------------------------------------------------------
; prototype:
;		LPVOID __stdcall AlignAlloc(size_t sizeAlloc);
;
; description:
;		this function will return memory which is aligned on a 16-byte memory
;		segment. having memory aligned gives a performance increase on most
;		processors plus a lot of the newer instruction sets (MMX, SSE, SSE2,
;		SSE3 and 3DNow will throw exceptions if the memory isn't properly
;		aligned. all memory is NULLed out.
;
; returns:
;		NULL = not enough available physical memory (virtual still may be
;			   available -- try VirtualAlloc)
;		a pointer to the memory which is aligned a 16-byte memory segment.
; ----------------------------------------------------------------------------
align 16
_fmAlloc@4 proc public
	; get our allocation size parameter
	mov ecx, [esp + 4]
	
	; adjust the size of the allocation so we can align it on the correct
	; memory segment offset. we are also going to add some extra space for
	; the structure which keeps the 'real' allocation information.
	; the value of 23 is 8 bytes to store the 'real' memory pointer plus
	; allocation size. the other 15 bytes are to allow for aligning the
	; returned pointer on a 16-byte memory offset
	add ecx, 23 
	
	; get our requested memory from the OS
	push ecx
	push HEAP_ZERO_MEMORY	; HEAP_ZERO_MEMORY
	push [hProcessHeap]
	call HeapAlloc
	mov ecx, eax
	
	; make sure we allocated the requested amount plus some extra
	test eax, eax
	je short @F	; didn't
	
	; we have, so lets update some internal references and return
	; the correct alignment position into the memory.
	; aligned_ptr = ((memory_ptr + ALIGNMENT_ADJUST + sizeof(ABH)) & ~ALIGNMENT_ADJUST);	
	lea eax, [ecx + 23]
	mov edx, [esp + 4]
	and eax, -16
	
	mov dword ptr [eax-8], ecx		; AlignBlockHeader.Memory (but one ABH size back)
	mov dword ptr [eax-4], edx		; keep track of 'requested memory size'
	
@@:
	ret 4
_fmAlloc@4 ENDP

END