TITLE "Reallocates a block of memory"

.586
.model flat             ; memory model
option casemap :none    ; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
include Kernel32.inc


; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
include asm_Memory.inc



.code

align 16
; ----------------------------------------------------------------------------
; Prototype:
;   LPVOID __stdcall fmResize(const void* pMemory, size_t newSize);
;
; Description:
;		Re-Allocates a segment of memory using HeapReAlloc.  This will keep the
;		original contents of the memory plus add in new space. The reallocation
;		is also aligned a 16-byte segment.
;
;  NOTE:
;    the new return address may or may not be the same as the pointer passed
;    in. Some something like the following will leak memory:
;		void* pBuf = ResizeMemory(pBuf, 0xffffffff);  
;
;
; Returns:
;		Pointer to the new allocated memory or NULL if failed.
; ----------------------------------------------------------------------------
_fmResize@8 proc public
	; get our allocation size parameter
	mov eax, [esp + 4]
	mov ecx, [esp + 8]
	
	; adjust the size of the allocation so we can align it on the correct
	; memory segment offset. we are also going to add some extra space for
	; the structure which keeps the 'real' allocation information.
	add ecx, 23		;	ALIGNMENT_ADJUST + sizeof(AlignBlockHeader) 
	
	; not a null pointer -- need to get the correct offset of the buffer so
	; we can get the orginal memory pointer	
	and eax, -4
	
	; get the 'real' starting pointer of the memory to be freed
	mov eax, [eax - 8]

	; get our requested memory from the OS
	push ecx
	push eax
	push HEAP_ZERO_MEMORY
	push [hProcessHeap]
	call HeapReAlloc
	mov ecx, eax
	
	; make sure we allocated the requested amount plus some extra
	test eax, eax
	je short @F	; didn't
	
	; we have, so lets update some internal references and return
	; the correct alignment position into the memory.
	; aligned_ptr = ((memory_ptr + ALIGNMENT_ADJUST + sizeof(ABH)) & ~ALIGNMENT_ADJUST);	
	lea eax, [ecx + 23]
	mov edx, [esp + 8]
	and eax, -16
	
	mov dword ptr [eax-8], ecx		; AlignBlockHeader.Memory (but one ABH size back)
	mov dword ptr [eax-4], edx		; keep track of 'requested memory size'
	
@@:
	ret 8
_fmResize@8 endp

END