TITLE "Registry Reading"

.486
.model flat
option casemap :none      ; case sensitive

extern __imp__RegOpenKeyExA@20:near
extern __imp__RegCloseKey@4:near
extern __imp__RegQueryValueExA@24:near

.code
align 16
; ----------------------------------------------------------------------------
; prototype:
;   LONG RegRead(HKEY Root, LPCTSTR Key, LPCTSTR Field, BYTE* Data, int DataLen);
;
; description:
;   Opens the specified registry key of Root\Key
;   Reads in the field/value data
;   Closes the registry key after done reading
;
; ESP +  0 = return instruction pointer (used by RET)
; ESP +  4 = EBX preservation
;
; ESP +  8 = Root parameter
; ESP + 12 = Key parameter
; ESP + 16 = Field parameter
; ESP + 20 = Data parameter (where data is stored)
; ESP + 24 = DataLen parameter
; ============================================================================
_RegReadA@20 proc near
	; preserve the EBX register
	push ebx
	
	; load registers with parameters
	lea eax, [esp + 12]
	mov edx, [esp + 12]
	mov ecx, [esp + 8]
		
	; clear out ebx and use this value when calling APIs (NULL) -- this is
	; only to shorten the ASM code bytes (instead of 4 bytes per NULL we
	; are only using one byte for EBX).
	xor ebx, ebx

	; try and open the specified registry key by calling the Win32 API call
	push eax
	push 020019h		; KEY_READ
	push ebx
	push edx
	push ecx
	call dword ptr __imp__RegOpenKeyExA@20
	
	; make sure we opened it
	test eax, eax
	jne @F
	
	; load registers with parameters
	lea eax, [esp + 24]
	mov ecx, [esp + 16]
	mov edx, [esp + 20]
	
	; make the win32 api call now to read the data from registry
	push eax
	push edx
	push ebx
	push ebx
	push ecx
	push [esp + 32]
	call dword ptr __imp__RegQueryValueExA@24
	
	; save the API result
	mov ebx, eax
	
	; close the registry key
	push [esp + 12]
	call dword ptr __imp__RegCloseKey@4
	
	; get our result back into EAX to return back to callee.
	mov eax, ebx
	
@@:
	pop ebx
	
	ret 20
_RegReadA@20 endp

END
