TITLE "Registry Reading"

.486
.model flat
option casemap :none      ; case sensitive

extern __imp__RegOpenKeyExA@20:near
extern __imp__RegCloseKey@4:near
extern __imp__RegQueryValueExA@24:near

REGISTRY_PARAMS struct
	field		dword ?		(pointer)
	value		dword ?		(pointer)
	lenVal	dword ?
	unused	dword ?
REGISTRY_PARAMS ends

.code
align 16
; ----------------------------------------------------------------------------
; prototype:
;   LONG RegReadEx(HKEY Root, LPCTSTR Key, REGISTRY_PARAMS* Recs, int RecCount);
;
; description:
;	Opens the specified registry key of Root\Key
;   Reads in the field/value data for the number of entries specified.
;	Closes the registry key after done reading (either number of recs or error)
;
; ESP +  4 = Root parameter
; ESP +  8 = Key parameter
; ESP + 12 = Recs parameter (array)
; ESP + 16 = Recs count parameter
; ============================================================================
_RegReadExA@16 proc near
	; try and open the specified registry key before we get much farther
	; into the function. no point is preserving registers, etc if we can't
	; open the key.
	lea eax, [esp + 8];
	mov ecx, [esp + 8];
	mov edx, [esp + 4];
	
	; call the win32 api registry open function
	push eax
	push 020019h		; KEY_READ
	push 0
	push ecx
	push edx
	call dword ptr __imp__RegOpenKeyExA@20
	
	; make sure we opened it
	test eax, eax
	jne Done
	
	; preserve our registers now
	push ebx
	push esi
	push edi
	push ebp

	; set up for the looping of the records to read into
	xor edi, edi	; loop counter
	mov ebp, [esp + 24]
	mov esi, [esp + 28]
	mov ebx, [esp + 32]

align 4
ReadLoop:
	; get the information out of the registry
	lea eax, [esi].REGISTRY_PARAMS.lenVal
	mov ecx, [esi].REGISTRY_PARAMS.field
	mov edx, [esi].REGISTRY_PARAMS.value
	
	; update the loop counter
	add edi, 1
	add esi, sizeof REGISTRY_PARAMS

	; make the win32 api call now	
	push eax
	push edx
	push 0
	push 0
	push ecx
	push ebp
	call dword ptr __imp__RegQueryValueExA@24
	
	; test to make sure we read it ok
	test eax, eax
	jne ExitLoop
	cmp edi, ebx
	jl ReadLoop
	
ExitLoop:
	; save our eax register for the return value
	mov ebx, eax
	
	; close the registry key
	push ebp
	call dword ptr __imp__RegCloseKey@4

	; restore the registers we need to
	mov eax, ebx
	pop ebp
	pop edi
	pop esi
	pop ebx	
	
Done:
	ret 16
_RegReadExA@16 endp

END
