// ---------------------------------------------------------------------------
// Miscellaneous.h
// ===========================================================================
#if !defined(AFX_ASM_MISC_H__A9DB83DB_A9FD_11D0_BFD1_444553540000__INCLUDED_)
#define AFX_ASM_MISC_H__A9DB83DB_A9FD_11D0_BFD1_444553540000__INCLUDED_


#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


struct REGISTRY_PARAMS
{
	LPCTSTR field;
	BYTE*   Data;
	DWORD   DataLen;
	void*	  reserved;
};


#if defined(__cplusplus)
extern "C" {
#endif

// ---------------------------------------------------------------------------
// Description:
//		This function returns a 32-bit integer containing information about the
//		type of microprocessor and the available instruction set. This function
//		also checks if XMM registers are enabled by the operating system.
//		Each feature is indicated by a bit in the returned value.
//
// Returns:
//		The return value is 0 if microprocessor has no CPUID instruction
//		bits 0-3 = model
//		bits 4-7 = family:  4 for 80486, Am486, Am5x86
//		                    5 for P1, PMMX, K6
//		                    6 for PPro, P2, P3, Athlon, Duron
//		                   15 for P4, Athlon64, Opteron
//		bit    8 = vendor is Intel
//		bit    9 = vendor is AMD
//		bit   11 = XMM registers enabled by operating system
//		bit   12 = floating point instructions supported
//		bit   13 = time stamp counter supported
//		bit   14 = CMPXCHG8 instruction supported
//		bit   15 = conditional move and FCOMI supported (PPro, P2, P3, P4, Athlon, Duron, Opteron)
//		bit   23 = MMX instructions supported (PMMX, P2, P3, P4, K6, Athlon, Duron, Opteron)
//		bit   24 = FXSAVE / FXRSTOR supported
//		bit   25 = SSE instructions supported (P3, P4, Athlon64, Opteron)
//		bit   26 = SSE2 instructions supported (P4, Athlon64, Opteron)
//		bit   27 = SSE3 instructions supported (forthcoming "Prescott")
//		bit   28 = hyperthreading (P4)
//
//		AMD-specific instructions:
//		bit   29 = 3DNow instructions
//		bit   30 = 3DNow extension instructions
//		bit   31 = MMX extension
// ---------------------------------------------------------------------------
int __stdcall DetectProcessor();


// ---------------------------------------------------------------------------
// Description:
//		This function returns an integer indicating which instruction set is
//		supported by the microprocessor and operating system. A program can
//		call this function to determine if a particular set of instructions can
//		be used.
// ---------------------------------------------------------------------------
int __stdcall InstructionSet();


// ---------------------------------------------------------------------------
// Description:
//		This function produces a zero-terminated ASCII string containing a name
//		for the microprocessor in Human readable format.  The buffer size must
//		be at LEAST 68 bytes to hold the full length including a NULL.
// ---------------------------------------------------------------------------
void __stdcall ProcessorName(
	OUT char* text);


// ---------------------------------------------------------------------------
// Description:
//		A very quick function to convert a double precision floating point
//		number to an integer, rounding to the nearest or even whole number.
//		Does not check for overflow.
//
// NOTE:
//		This function is *much* faster then the default conversion method in
//		C/C++ which uses truncation.
//
// Returns:
//		A whole number rounded to the nearest or even number.
// ---------------------------------------------------------------------------
int _stdcall Round(IN const double x);


// ---------------------------------------------------------------------------
// Description:
//		Returns the value of the time stamp counter which counts clock cycles.
//		To count how many clock cycles a piece of code will take, call ReadClock()
//		before and after the code to measure and calcuate the difference.
//		This function itself takes approximately 700 clock cycles ona Pentium 4
//		and approximately 225 clock cycles on a Pentium II and Pentium III.
//
// NOTE:
//		Does NOT work on a 80386 or 80486 chipset.
//
// Returns:
//		The current CPU cycle count.
// ---------------------------------------------------------------------------
int __stdcall ReadClock();


LONG RegReadA(HKEY Root, LPCTSTR Key, LPCTSTR Field, BYTE* Data, DWORD DataLen);
LONG RegReadExA(HKEY Root, LPCTSTR Key, REGISTRY_PARAMS* Recs, int RecCount);


#if defined(__cplusplus)
}
#endif


// ===========================================================================
#endif // #define AFX_ASM_MISC_H__A9DB83DB_A9FD_11D0_BFD1_444553540000__INCLUDED_
