TITLE "Removes whitespace characters from right side of string"

.586
.model flat             ; memory model
option casemap :none    ; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
; (none)

; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
include asm_Strings.inc


.code
; ----------------------------------------------------------------------------
; prototype:
;		void __stdcall RightTrimWhiteSpaceA(char* pString);
;
; Description:
;   ANSI version to remove the following three characters (tab, space, quote)
;   from the right side of the string. Once we find a character that is not one
;   of those characters, it'll stop and place a NULL to terminate it there.
;
; Return value:
;		(none)
;
; ----------------------------------------------------------------------------
; variable offests into ESP upon function entering and adjustment of the
; stack for local variables and register preservation
; ----------------------------------------------------------------------------
; ESP + 0 = (return address)
; ESP + 4 = pString parameter
; ============================================================================
align 16
_RightTrimWhiteSpaceA@4 proc public
    ; get the length of the string
    push [esp + 4]
    call strLengthA
    
    ; get our parameter again into a register
    mov edx, [esp + 4]

    ; are we at the beginning?
    test eax, eax
    je short TrimDone
    
align 4
FindTrimOffset:
    ; check to see if the follow character is on of three types:
    ; Tab, Space or a Quote
    mov cl, byte ptr [edx+eax-1]    ; zero based
    cmp cl, 9   ; tab character
    je short AdjustOffset
    cmp cl, 32  ; space character
    je short AdjustOffset
    cmp cl, 34  ; quote character
    jne short TrimDone
    
align 4
AdjustOffset:
    ; move to the next character from the right side (going left)
    sub eax, 1
    jnz short FindTrimOffset
    
align 4    
TrimDone:
    ; Null terminate this string at the offset (even if it already has one)
    mov byte ptr [edx+eax], 0
    
    ; return back to the caller
    ret 4
_RightTrimWhiteSpaceA@4 ENDP

END
