TITLE "Will convert all characters to uppercase or lowercase"

.586
.model flat			; memory model
option casemap:none	; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
; (none)

; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
include asm_Strings.inc

.code
; ----------------------------------------------------------------------------
; prototype:
;		void __stdcall strCaseA(char* pString, const enumStringCaseType eType);
;
; Description:
;		ANSI version to upper or lower case a string in-place.
;
; Return value:
;		passed in string
;
; ----------------------------------------------------------------------------
; variable offests into ESP upon function entering and adjustment of the
; stack for local variables and register preservation
; ----------------------------------------------------------------------------
; ESP + 0  = EBX registered saved
; ESP + 4  = (return address)
; ESP + 8  = pString parameter
; ESP + 12 = Upper or Lower type parameter
; ============================================================================
align 16
_strCaseA@8 proc public
    ; save the EBX register
    push ebx
    
    ; get our string to work with and the type
    mov ecx, [esp + 12]
    mov eax, [esp + 8]
    mov edx, 32 ; start with lower case value

    ; are we doing upper or lower case conversion?
    sub eax, 1
    cmp ecx, StringCaseLower
    mov ebx, 65     ; upper 'A'
    mov ecx, 90     ; upper 'Z'
    je short DoConversion
    
    ; change to upper case checking
    mov ebx, 96     ; lower 'a'
    mov ecx, 127    ; lower 'z'
    not edx
    add edx, 1

align 4    
DoConversion:
     ; advance the index into the string
    add eax, 1

    ; are we at the end of the string yet?
    cmp byte ptr [eax], 0
    je short Finished
    
    ; check to see if it needs to be converted
    cmp byte ptr [eax], bl
    jb short DoConversion
    cmp byte ptr [eax], cl
    ja short DoConversion
    
    ; convert it to upper case
    add byte ptr [eax], dl
    jmp short DoConversion
    
align 4
Finished:
    ; restore the EBX register
    pop ebx
    
	; return back to caller
	mov eax, [esp + 8]
	ret 8
_strCaseA@8 ENDP

; ----------------------------------------------------------------------------
; prototype:
;		char* __stdcall strLowerA(char* pString);
;
; Description:
;		ANSI version to lower case a string in-place.
;
; Return value:
;		passed in string
;
; ----------------------------------------------------------------------------
; variable offests into ESP upon function entering and adjustment of the
; stack for local variables and register preservation
; ----------------------------------------------------------------------------
; ESP + 0 = (return address)
; ESP + 4 = pString parameter
; ============================================================================
align 16
_strLower@4 proc public
    ; call the conversion function
    push StringCaseLower
    push [esp + 8]
    call strCaseA    
    
    ; get our parameter in EAX for returning
    mov eax, [esp + 4]
    
    ; return back to the caller
    ret 4
_strLower@4 ENDP

; ----------------------------------------------------------------------------
; prototype:
;		char* __stdcall strUpperA(char* pString);
;
; Description:
;		ANSI version to upper case a string in-place.
;
; Return value:
;		passed in string
;
; ----------------------------------------------------------------------------
; variable offests into ESP upon function entering and adjustment of the
; stack for local variables and register preservation
; ----------------------------------------------------------------------------
; ESP + 0 = (return address)
; ESP + 4 = pString parameter
; ============================================================================
align 16
_strUpper@4 proc public
    ; call the conversion function
    push StringCaseUpper
    push [esp + 8]
    call strCaseA    
    
    ; get our parameter in EAX for returning
    mov eax, [esp + 4]
    
    ; return back to the caller
    ret 4
_strUpper@4 ENDP

END
