TITLE "Copy a string from one memory location to another"

.586
.model flat			; memory model
option casemap:none	; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
; (none)

; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
; (none)


.code

; ----------------------------------------------------------------------------
; prototype:
;		void __stdcall strCopyA(char* pDestString, const char* pSrcString);
;
; Description:
;		ANSI version to copy Source string (including NULL) into Destination
;
; Return value:
;		(none)
;
; ----------------------------------------------------------------------------
; variable offests into ESP upon function entering and adjustment of the
; stack for local variables and register preservation
; ----------------------------------------------------------------------------
; ESP + 0  = EBX register saved value
; ESP + 4  = (return address)
; ESP + 8  = Dest parameter
; ESP + 12 = Src parameter
; ============================================================================
align 16
_strCopyA@8 proc public
	push ebx
	
	; check the source and destination strings to make sure they aren't NULL
	; pointers
	mov eax, [esp + 12]
	mov ebx, [esp + 8]
	
	test eax, eax		; check source parameter
	je Done
	test ebx, ebx		; check destination parameter
	je Done	
	
; start searching the string for the first occurance of a NULL value
; we'll read four-bytes at a time.
align 4
Find4Null:
	mov edx, [eax]		; get the first DWORD to search
	
	; this will set up the test pattern and also set any flags which will signal
	; that one or more of the bytes in the DWORD is a NULL value.
	lea ecx, [edx - 01010101h]
	not edx
	and edx, 80808080h
	and edx, ecx
	
	; if the flag is not set, we'll check the next DWORD
	jnz Find1Null
	
	; we didn't find the NULL in one of the DWORDs so just write the
	; value out in the destination
	mov ecx, [eax]
	add eax, 4			; increment the offset into the string
	mov [ebx], ecx
	add ebx, 4
	jmp Find4Null

; now that we have found a NULL terminator, we need to copy the remaining
; bytes including the NULL into the destination string
align 4
Find1Null:
	mov cl, byte ptr [eax]
	add eax, 1
	mov byte ptr [ebx], cl
	add ebx, 1
	test cl, cl
	jne Find1Null

Done:	
	; restore the EBX register used
	pop ebx

	; return back to caller
	ret 8

_strCopyA@8 ENDP

END
