TITLE "Finding the length of a NULL terminated string"

.586
.model flat			; memory model
option casemap:none	; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
; (none)

; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
; (none)


.code
; ----------------------------------------------------------------------------
; prototype:
;   int strLengthA(const char* pString);
;
; description:
;   Walk the string to find the NULL terminator at the end. This function
;   assumes you will always have a buffer size divisible by four. size of
;   the buffer is different then the length of the string contained in that
;   buffer.
;
; returns:
;   1 based index where it found the NULL terminator
;
; ----------------------------------------------------------------------------
; variable offests into ESP upon function entering and adjustment of the
; stack for local variables and register preservation
; ----------------------------------------------------------------------------
; ESP + 0 = (return address)
; ESP + 4 = pString parameter
; ============================================================================
align 16
_strLengthA@4 proc public
	mov ecx, [esp+4]	; string parameter
	xor eax, eax		; clear initial size
	
align 4
@@:
	mov edx, [ecx+eax]		; get the DWORD to search
	lea edx, [edx-1010101h] ; sub 1 from each byte in eax
	add eax, 4              ; ready for next dword
	and edx, 80808080h      ; check sign bit in all byte
	jz @B                  ; more to do until we have a sign

	; one of the four bytes has a zero in it, find out which one.        
	test edx, 000000FFh      ; is byte-1 zero?
	jnz Offset_Minus4
	test edx, 0000FF00h      ; is byte-2 zero?
	jnz Offset_Minus3
	test edx, 00FF0000h      ; is byte-3 zero?
	jnz Offset_Minus2
	
align 4	
Offset_Minus1:
	sub eax, 1
	ret 4
	
align 4
Offset_Minus2:
	sub eax, 2
	ret 4
	
align 4
Offset_Minus3:
	sub eax, 3
	ret 4
	
align 4
Offset_Minus4:
	sub eax, 4
	ret 4

_strLengthA@4 ENDP

END
