TITLE "Search for a sub-string within a string"

.586
.model flat				; memory model
option casemap:none	; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
; (none)

; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
; (none)


.code
; ----------------------------------------------------------------------------
; prototype:
;		char* __stdcall strSubA(const char* pString, const char* pSubStr);
;
; Description:
;		ANSI version to find the sub-string within the specified string
;
; Return value:
;		Offset of where it found it or NULL if it didn't.
; ----------------------------------------------------------------------------

; ----------------------------------------------------------------------------
; variable offests into ESP upon function entering and adjustment of the
; stack for local variables and register preservation
; ----------------------------------------------------------------------------
; ESP + 0 = (return address)
; ESP + 4 = pString parameter
; ESP + 8 = pSubStr parameter
; ============================================================================
align 16
_strSubA@8 proc public
    ; load our parameters before we adjust the stack (ESP)
	mov eax, [esp + 4]
	mov ecx, [esp + 8]       ; str2 (the string to be searched for)

	; preserve ebx, esi, edi and ebp registers
	push ebx
	push esi
	push edi
	push ebp

	mov edx, [ecx]		; edx contains up to four chars from str2
	mov ebp, ecx

	mov edi, eax		; str1 (the string to be searched)
	test dl, dl			; is str2 empty?
	jz not_found

; dl contains first char from str2
; dh contains second char from str2
; edi holds str1

align 4
findnext:
	mov esi, edi		; esi = edi = pointers to somewhere in str1
	mov ecx, ebp		; str2

align 4
loop_start:
	mov al, [esi]		; put next char from str1 into al
	add esi, 1			; increment pointer in str1
	
in_loop:
	cmp al, dl
	je first_char_found

	test al, al			; end of str1?
	jnz loop_start		; no, go get another char from str1

not_found:
	xor eax, eax

align 4        
done:
	pop ebp
	pop edi
	pop esi
	pop ebx
	ret 8


; recall that dh contains the second char from str2
align 4
first_char_found:
	mov al, [esi]				; put next char from str1 into al
	add esi, 1

	cmp al, dh					; compare second chars
	jnz in_loop					; no match, continue search

two_first_chars_equal:
	lea edi, [esi - 1]		; store position of last read char in str1

align 4
compare_loop:
	mov ah, [ecx + 2]			; put next char from str2 into ah
	test ah, ah					; end of str2?
	jz match						; if so, then a match has been found

	mov al, [esi]				; get next char from str1
	add esi, 2					; bump pointer into str1 by 2

	cmp al, ah					; are chars from str1 and str2 equal?
	jne findnext				; no

; do one more iteration
	mov al, [ecx + 3]			; put the next char from str2 into al
	test al, al					; end of str2
	jz match						; if so, then a match has been found

	mov ah, [esi - 1]			; get next char from str1
	add ecx, 2					; bump pointer in str1 by 2
	cmp al, ah					; are chars from str1 and str2 equal?
	je compare_loop

; no match. test some more chars
	jmp findnext

match:
	lea eax, [edi - 1]
	jmp done

_strSubA@8 ENDP

END
