TITLE "Concatenate a string to another string"

.586
.model flat			; memory model
option casemap:none	; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
; (none)

; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
include asm_Strings.inc


.code

; ----------------------------------------------------------------------------
; prototype:
;   wchar_t* __stdcall strCopyW(wchar_t* pDestString, const wchar_t* pSrcString);
;
; Description:
;   UNICODE version to copy Source string (including NULL) into Destination
;
; Return value:
;   (none)
;
; ----------------------------------------------------------------------------
; variable offests into ESP upon function entering and adjustment of the
; stack for local variables and register preservation
; ----------------------------------------------------------------------------
; ESP + 0  = EBX register saved
; ESP + 4  = (return address)
; ESP + 8  = Dest parameter
; ESP + 12 = Src parameter
; ============================================================================
align 16
_strCatW@8 proc public
    ; save the EBX register
    push ebx
    
    ; get the current length of our destination string
    push [esp + 8]
    call strLengthW
    
    ; double the value for byte count
    shl eax, 1 
    
    ; get the source parameter into EDX, dest into EBX
    mov edx, [esp + 12]
    mov ebx, [esp + 8]
    
    ; start at the end of dest string
    add ebx, eax
    
    ; walking forward through source
    xor eax, eax
    
CopyWhileNotNull:
    ; Get four characters from the source into ECX
    mov ecx, [edx + eax*2]

    ; Need to check the LoWord of the four bytes to make sure we haven't found
    ; the NULL terminator for this string. If we have, we can't write out all
    ; four bytes because the last two bytes may not belong to us and we'll
    ; cause an access violation.
    test ecx, 0ffffh
    jz short FoundNull

    ; didn't find the NULL so we can safely write all four bytes    
    mov [ebx + eax*2], ecx
        
    ; adjust offset and check for NULL in the HI word
    add eax, 2
    and ecx, 0ffff0000h
    jnz short CopyWhileNotNull
    
    ; we have to back up one to place our NULLs again
    add eax, -1
    
FoundNull:
    ; make sure to terminate our string
    mov [ebx + eax*2], word ptr 0
    
    ; restore the EBX register
    pop ebx
    
    ; return Dest parameter back in EAX
    mov eax, [esp + 4]
    ret 8    
_strCatW@8 endp

END
