TITLE "Compare two strings --- Case sensitive"

.586
.model flat			; memory model
option casemap:none	; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
; (none)

; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
; (none)


.code

; ----------------------------------------------------------------------------
; prototype:
;		int __stdcall strCompareW(const wchar_t* string1, const wchar_t* string2);
;
; Description:
;		UNICODE version to compare two null-terminated strings. This function
;		is case-sensitive.  IE: "My TEST" != "my Test"
;
; Returns:
;		< 0 = string1 < string2
;		  0 = string1 = string2
;		> 0 = string1 > string2
;
; ----------------------------------------------------------------------------
; variable offests into ESP upon function entering and adjustment of the
; stack for local variables and register preservation
; ----------------------------------------------------------------------------
; ESP + 0 = (return address)
; ESP + 4 = String1 parameter
; ESP + 8 = String2 parameter
; ============================================================================
align 16
_strCompareW@8 proc public
	; move the string parameters into registers to work with
	mov ecx, [esp + 4]	; String1 parameter
	mov edx, [esp + 8]	; String2 parameter

	; to not have the performance hit of using the high byte of a register
	; while we are checking byte-by-byte, we are going to use EBX so we must
	; preserve this register first.
	push ebx

	; Clear out the EAX register for the return value. We'll have the low byte
	; of EAX (al) with the correct return value once our loop is done.
	xor eax, eax
		
	; loop through the strings and see if they match by checking character
	; against character and ending when we find a NULL terminator in either
	; string or one of the characters doesn't match.
	
	; NOTE:
	;	A nice optimization to this routine will be to check DWORDs at a time
	;  until a difference is found and then check the DWORD to see where the
	;  byte character may be different.

align 4
@@:
	; get the byte from each string to compare
	mov al, byte ptr [ecx]
	mov bl, byte ptr [edx]
	
	; advance our string offsets
	add	ecx, 2
	add	edx, 2
	
	; check and see if character matches
	sub	al, bl
	jnz	@F
	
	; did we find the terminating NULL character?
	or bl, bl
	jnz	@B
	
@@:
	; return what we found in the difference	
	movsx eax, al	; need to set the signed flag.

	; restore our register
	pop ebx

	; return
	ret 8
_strCompareW@8 ENDP

END
