TITLE "Thread wide and Window message loop"

.586
.model flat
option casemap :none      ; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
include Windows.inc
include User32.inc


; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------



.code
align 16
; ----------------------------------------------------------------------------
; prototype:
;   LRESULT MsgLoopA(HWND hWnd);
;
; description (ANSI version):
;   Message loop for the main app (by specifying NULL for hWnd)
;   can be used for a window which needs another msg loop handler
;
; ESP +  0 = EBX preservation
; ESP +  4 = ESI preservation
; ESP +  8 = MSG structure on stack
; ESP + 44 = hWnd parameter
; ============================================================================
_MsgLoopA@4 proc near
	; make room on the stack for a MSG structure and pad the remaining four
	; bytes so it is nicely aligned to a 16-byte boundary
	sub esp, 32

	; preserve our registers
	push ebx		; addr of MSG structure
	push esi		; hWnd parameter

	; get the address of the MSG stack variable. this will be used in the calls
	; to GetMessage, TranslateMessage, DispatchMessage
	lea ebx, [esp + 8]

	; load the hWnd parameter into ESI for loop code
	mov esi, [esp + 44]
	
align 4
@@:
	; main message loop (thread or individual window)
	xor eax, eax
	push eax			; 0
	push eax			; 0
	push esi			; hWnd
	push ebx			; &Msg
	call GetMessageA
	
	; see if we got some kind of error message from GetMessage or if a
	; PostQuitMessage() was called and placed a WM_QUIT in the queue.
	; return values of GetMessage:
	; < 0 = error condition
	;   0 = WM_QUIT found (result of PostQuitMessage)
	; > 0 = normal msg to process
	test eax, eax
	jle @F
	
	; dispatch it now
	push ebx
	call TranslateMessage
	push ebx
	call DispatchMessageA
	jmp @B
	
@@:
	; restore the registers
	pop esi
	pop ebx

	add esp, 32	
	ret 12
_MsgLoopA@4 ENDP

END
