TITLE "Thread wide and Window message loop"

.586
.model flat
option casemap :none      ; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
include Windows.inc
include User32.inc


; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------



.code
align 16
; ----------------------------------------------------------------------------
; prototype:
;   LRESULT MsgLoopWAccelA(HWND hWnd, HINSTANCE hInst, LPCSTR AccelID);
;
; description (ANSI version):
;   Message loop for the main app (by specifying NULL for hWnd)
;   can be used for a window which needs another MsgLoop handler
;
; ESP +  0 = EBX preservation
; ESP +  4 = ESI preservation
; ESP +  8 = EDI preservation
;
; ESP + 12 = MSG structure on stack
; ESP + 40 = ACCEL handle if loaded

; ESP + 48 = hWnd parameter
; ESP + 52 = hInst parameter
; ESP + 56 = AccelID parameter
; ============================================================================
_MsgLoopWAccelA@12 proc near
	; make room on the stack for a MSG structure, ACCEL handle and also nicely
	; align the stack size to be 16-byte divisible. this isn't aligning on a
	; 16-byte boundary, just making the stack size divisible by 16
	sub esp, 32

	; preserve our registers
	push ebx		; addr of MSG structure
	push esi		; accelerator handle value
	push edi		; hWnd parameter

	; load the parameters into registers
	mov edi, [esp + 48]		; hwnd Parameter
	mov eax, [esp + 52]		; hInstance parameter
	mov ecx, [esp + 56]		; AccelID parameter
	
	; get the address of the MSG stack variable. this will be used in the calls
	; to GetMessage, TranslateAccelerator, TranslateMessage, DispatchMessage
	lea ebx, [esp + 12]
	
	; intially set the handle of accelerator table to NULL
	xor esi, esi
	
	; load the accelerator
	push ecx
	push eax
	call LoadAcceleratorsA
	mov esi, eax

align 4
MsgLoop:
	; main message loop (thread or individual window)
	xor eax, eax
	push eax			; 0
	push eax			; 0
	push edi			; hWnd
	push ebx			; &Msg
	call GetMessageA

	; see if we got some kind of error message from GetMessage or if a
	; PostQuitMessage() was called and placed a WM_QUIT in the queue.
	; return values of GetMessage:
	; < 0 = error condition
	;   0 = WM_QUIT found (result of PostQuitMessage)
	; > 0 = normal msg to process
	test eax, eax
	jle Cleanup
	
	; since we have received a message, we need to see if this message
	; is from any accelerators.
	push ebx
	push esi
	push edi
	call TranslateAcceleratorA
	
	; if the message was from an accelerator and it was processed,
	; we shouldn't do any more processing on this message.
	test eax, eax
	jne MsgLoop
	
	; since we didn't handle the message, we need to dispatch it now
	push ebx
	call TranslateMessage
	push ebx
	call DispatchMessageA
	jmp MsgLoop
	
Cleanup:
	; free our accelerator handle resource if we have loaded it
	push esi
	call DestroyAcceleratorTable
	
@@:
	; restore the registers
	pop edi
	pop esi
	pop ebx

	add esp, 32	
	ret 12
_MsgLoopWAccelA@12 ENDP

END
