TITLE "Thread wide and Window message loop"

.586
.model flat
option casemap :none      ; case sensitive
option readonly


; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
include Windows.inc
include User32.inc


; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------


.code
align 16
; ----------------------------------------------------------------------------
; prototype:
;   ATOM RegClassA(HINSTANCE hInst, HICON hIcon, HCURSOR hCursor, LPCSTR className, WNDPROC wndProc);
;
; description (ANSI version):
;   Registers a window class type with the operating system
;
; ESP +  0 = EBP preservation
; ESP +  4 = EDI preservation
; ESP +  8 = ESI preservation
; ESP + 12 = EBX preservation
;
; ESP + 16 = WNDCLASSEX structure on stack
;
; ESP + 68 = hInst parameter
; ESP + 72 = hIcon parameter
; ESP + 76 = hCursor parameter
; ESP + 80 = className parameter
; ESP + 84 = wndProc parameter
; ============================================================================
_RegClassA@20 PROC NEAR
	; make room on the stack for a WNDCLASSEX structure plus preservation of registers
	sub esp, 64

	; preserve our critical registers
	mov [esp +  0], ebp
	mov [esp +  4], edi
	mov [esp +  8], esi
	mov [esp + 12], ebx
	
	; clear out the structure
	lea edi, [esp + 16]	; get the address
	xor eax, eax
	mov ecx, sizeof WNDCLASSEX / 4
	rep stosd
	
	; get the parameters into registers
	mov eax, [esp + 68]		; hinst
	mov ebx, [esp + 72]		; hicon
	mov esi, [esp + 76]		; hcursor
	mov ecx, [esp + 80]		; className
	mov edx, [esp + 84]		; wndProc
	
	; go back to the address of the WNDCLASSEX structure so we don't have to do
	; a 'lea' again. we need the address for the RegisterClassEx() call.
	sub edi, sizeof WNDCLASSEX
	
	; fill in the values of the WNDCLASSEX structure that we care about
	mov dword ptr [esp+16].WNDCLASSEX.cbSize, sizeof WNDCLASSEX
	mov dword ptr [esp+16].WNDCLASSEX.style, CS_CLASSDC or CS_DBLCLKS or CS_HREDRAW or CS_PARENTDC or CS_VREDRAW
	mov [esp+16].WNDCLASSEX.lpfnWndProc, edx
	mov [esp+16].WNDCLASSEX.hInstance, eax
	mov [esp+16].WNDCLASSEX.hIcon, ebx
	mov [esp+16].WNDCLASSEX.hCursor, esi
	mov [esp+16].WNDCLASSEX.lpszClassName, ecx
	mov [esp+16].WNDCLASSEX.hIconSm, ebx

	; register the class type
	push edi
	call RegisterClassExA
		
	; restore the registers
	mov ebp, [esp +  0]
	mov edi, [esp +  4]
	mov esi, [esp +  8]
	mov ebx, [esp + 12]

	add esp, 64	
	ret 20
_RegClassA@20 ENDP

END
