TITLE	ZipCentral.asm

.586
.model flat, stdcall      ; memory model & calling convention
option casemap :none      ; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------


; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
include asm_Memory.inc
include asm_Analysis.inc

include asm_Zip.inc		; structures, defines and prototypes for the ZIP library


.const
align 16
CentralDirectorySignature       db 050h, 04bh, 05h, 06h
CentralDirectoryExtSignature    db 050h, 04bh, 07h, 08h


; NOTE:
;		The responsibility of freeing of the buffer created in zdOpenDirecotry
;		is handled by the caller (located in zaCloseArchive) to avoid a function
;		call into this module that contains a single invoke statment to HeapFree.

.code
align 16
; ----------------------------------------------------------------------------
; prototype:
; int __stdcall compareSort(void* pElem1, void* pElem2);
; ----------------------------------------------------------------------------
compareSortW proc Elem1:dword, Elem2:dword
	mov	eax, [Elem1]
	mov	ecx, [Elem2]
	
	mov	eax, [eax]
	mov	ecx, [ecx]

	; compare the two strings
	mov	edx, [eax].zeInstance.FileName
	mov	ecx, [ecx].zeInstance.FileName
		
align 4
@@:
	mov	al, [edx]
	mov	ah, [ecx]
	
	add	edx, 2
	add	ecx, 2
	sub	al, ah
	jnz	@F
	or ah, ah
	jnz	@B
	
@@:
	movsx eax, al

	ret
compareSortW endp


align 16
; ----------------------------------------------------------------------------
; prototype:
; BOOL __stdcall zdOpenDirectory(zdInstance* Instance, zfInstance* IO);
; ----------------------------------------------------------------------------
zdOpenDirectory proc uses ebx esi edi Instance:dword, IO:dword
	; start at the Central Directory for reading.  last XXX bytes of file.
	; save the current memory position pointer into the file and
	; make sure the signature matches correctly.
	mov	ecx, [IO]
	mov	edx, dword ptr CentralDirectorySignature
	lea	eax, [ecx].zfInstance.MemBuffer
	add	eax, [ecx].zfInstance.MemBufSize
	add	eax, -sizeof CentralDirSignature
	cmp dword ptr [eax].CentralDirSignature.Signature, edx
	je short @F
	
	mov	eax, ZipNoCentralDir
	ret
	
@@:
	; verify that we have entries and the signature sizes are valid.
	mov	edx, [eax].CentralDirSignature.dwSize
	movzx ebx, word ptr [eax].CentralDirSignature.wNumEntries

	.if (!edx && ebx) || (!ebx && edx)
		mov	eax, ZipBadZipFile
		ret
	.endif

	.if (!edx && !ebx)
		mov	eax, ZipDirEmpty
		ret
	.endif
	
	; go the the Central Directory offset in the file for reading each of the
	; file entries
	mov	edx, [eax].CentralDirSignature.dwOffset
	lea	eax, [ecx].zfInstance.MemBuffer
	add	eax, edx		; positive value
	mov	[ecx].zfInstance.MemPosition, eax
	
	; set up the array size/count and allocate memory to hold all the entries
	; which will be sorted in alphabetical order based on their filenames.  The
	; first section of the buffer will be indexes into the entries which are in
	; the last part of the buffer allocated.

	; array size = (NumEntries * sizeof(zeInstance)) + (NumEntries * sizeof(dword)) + sizeof(dword)
    mov eax, sizeof zeInstance 
    imul eax, ebx      ; NumEntries * sizeof(zeInstance) 
    lea	ecx, [ebx*4+eax+4]
    push ecx
    call fmAlloc

	; save our allocation and array count
    mov ecx, [Instance] 
    mov [eax].zdInstance.ArrayCount, ebx 
    mov [ecx], eax 

	; set up registers for looping to process each of the array elements
	; ebx = counter
	; esi = index into array to store Element (stores edi)
	; edi = address of element array
    lea esi, [eax+4] 
    lea edi, [eax+4+ebx*4] 

	; now read each of the file entries into the buffer -- this is just
	; their directory record information and not the contents of the file
@@: 
	mov [esi], edi
	push [IO]
	push edi
	call zeReadW 
	test eax, eax 
	je short @Failed 
	
	add edi, sizeof zeInstance 
	add esi, 4 
	sub ebx, 1 
	jnz	short @B

	; sort our array indexes using qsort with the filename of each entry
	; in ascending order.
	mov	edx, [Instance]
	mov	edx, [edx]
	lea	ecx, [edx].zdInstance.Array
	
	; call the sorting routine
	push offset flat:compareSortW
	push [edx].zdInstance.ArrayCount
	push ecx
	call QuickSort
	
	; success
	mov	eax, zipNoError
	ret
	
@Failed:
	mov	eax, ZipBadZipFile
	ret
zdOpenDirectory endp

END
