TITLE	zipEntry.asm

.586
.model flat, stdcall      ; memory model & calling convention
option casemap :none      ; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------


; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
include asm_Memory.inc
include asm_Zip.inc		; structures, defines and prototypes for the ZIP library


.const

align 16
ConvertString_Table DB 000H, 001H, 002H, 003H, 004H, 005H, 006H, 007H, 008H, 009H, 00aH, 00bH, 00cH, 00dH, 00eH, 00fH
   DB 010H, 011H, 012H, 013H, 014H, 015H, 016H, 017H, 018H, 019H, 01aH, 01bH, 01cH, 01dH, 01eH, 01fH
   DB 020H, 021H, 022H, 023H, 024H, 025H, 026H, 027H, 028H, 029H, 02aH, 02bH, 02cH, 02dH, 02eH, 05cH
   DB 030H, 031H, 032H, 033H, 034H, 035H, 036H, 037H, 038H, 039H, 03aH, 03bH, 03cH, 03dH, 03eH, 03fH
   DB 040H, 061H, 062H, 063H, 064H, 065H, 066H, 067H, 068H, 069H, 06aH, 06bH, 06cH, 06dH, 06eH, 06fH
   DB 070H, 071H, 072H, 073H, 074H, 075H, 076H, 077H, 078H, 079H, 07aH, 05bH, 05cH, 05dH, 05eH, 05fH
   DB 060H, 061H, 062H, 063H, 064H, 065H, 066H, 067H, 068H, 069H, 06aH, 06bH, 06cH, 06dH, 06eH, 06fH
   DB 070H, 071H, 072H, 073H, 074H, 075H, 076H, 077H, 078H, 079H, 07aH, 07bH, 07cH, 07dH, 07eH, 07fH
   DB 080H, 081H, 082H, 083H, 084H, 085H, 086H, 087H, 088H, 089H, 08aH, 08bH, 08cH, 08dH, 08eH, 08fH
   DB 090H, 091H, 092H, 093H, 094H, 095H, 096H, 097H, 098H, 099H, 09aH, 09bH, 09cH, 09dH, 09eH, 09fH
   DB 0a0H, 0a1H, 0a2H, 0a3H, 0a4H, 0a5H, 0a6H, 0a7H, 0a8H, 0a9H, 0aaH, 0abH, 0acH, 0adH, 0aeH, 0afH
   DB 0b0H, 0b1H, 0b2H, 0b3H, 0b4H, 0b5H, 0b6H, 0b7H, 0b8H, 0b9H, 0baH, 0bbH, 0bcH, 0bdH, 0beH, 0bfH
   DB 0c0H, 0c1H, 0c2H, 0c3H, 0c4H, 0c5H, 0c6H, 0c7H, 0c8H, 0c9H, 0caH, 0cbH, 0ccH, 0cdH, 0ceH, 0cfH
   DB 0d0H, 0d1H, 0d2H, 0d3H, 0d4H, 0d5H, 0d6H, 0d7H, 0d8H, 0d9H, 0daH, 0dbH, 0dcH, 0ddH, 0deH, 0dfH
   DB 0e0H, 0e1H, 0e2H, 0e3H, 0e4H, 0e5H, 0e6H, 0e7H, 0e8H, 0e9H, 0eaH, 0ebH, 0ecH, 0edH, 0eeH, 0efH
   DB 0f0H, 0f1H, 0f2H, 0f3H, 0f4H, 0f5H, 0f6H, 0f7H, 0f8H, 0f9H, 0faH, 0fbH, 0fcH, 0fdH, 0feH, 0ffH

align 4
EntrySignature      db 050h, 04bh, 01h, 02h


.code
align 16
; ----------------------------------------------------------------------------
; prototype:
; BOOL __stdcall zeRead(ZipEntry* pEntry, StorageIO* pStorage);
; ----------------------------------------------------------------------------
zeReadW proc private uses ebx esi edi Instance:dword, IO:dword
	mov	ebx, offset flat:EntrySignature
	mov	edx, [IO]
	mov	edi, [Instance]

	; read in the header from the zip file
	mov	eax, [edx].zfInstance.MemPosition
	mov	ecx, [ebx]
	mov	[edi].zeInstance.CDHdr, eax
	add	[edx].zfInstance.MemPosition, sizeof CentralDirectoryHeader

	; validate our signature with that from the Central Directory
	movzx	ebx, word ptr [eax].CentralDirectoryHeader.Common.FileNameSize 
	cmp	ecx, dword ptr [eax].CentralDirectoryHeader.Signature
	jne	short Failed
	
	; see if we have a filename
	test ebx, ebx
	je short @F
	
	; get a pointer to the filename
	mov	esi, [edx].zfInstance.MemPosition
	add	[edx].zfInstance.MemPosition, ebx
	
	; convert the filename from a non-null terminated to a null-terminated string
	add	ebx, 1
	shl ebx, 1  ; need to double so we have an accurate byte count
	push ebx
	call fmAlloc
	shr ebx, 1  ; get back our original length
	sub	ebx, 2

	mov	[edi].zeInstance.FileName, eax
	
align 4
; converts characters to lower case, converts from ansi to unicode
ConvertString:
	movzx edx, byte ptr [esi+ebx]
	mov	cl, byte ptr ConvertString_Table[edx]
	sub	ebx, 1
	mov	byte ptr [eax+ebx*2+2], cl
	jns	short ConvertString

	mov	esi, [edi].zeInstance.CDHdr
	
@@:
	; get ready to check the Extra Field and Comment to see if we are skipping past
	mov	ecx, [IO]
	movzx ebx, word ptr [esi].CentralDirectoryHeader.CommentSize
	movzx eax, word ptr [esi].CentralDirectoryHeader.Common.ExtraField
	mov	edx, [ecx].zfInstance.MemPosition
	
	; skip pass any extra field information
	test eax, eax
	je short @F

	; begin at the current memory position pointer
	add	edx, eax
	mov	[ecx].zfInstance.MemPosition, edx

@@:
	; skip pass any comment information
	test ebx, ebx 
	je short Success

	; begin at the current memory position pointer
	add	edx, ebx
	mov	[ecx].zfInstance.MemPosition, edx

Success:	
	; success
	mov	eax, 1
	ret
		
Failed:
	xor	eax, eax
	ret
zeReadW endp

END
