TITLE ""

.586
.model flat             ; memory model & calling convention
option casemap :none    ; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------


; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
include asm_Analysis.inc
include asm_Zip.inc

.code

; ----------------------------------------------------------------------------
; prototype:
;   (UNICODE) wchar_t* __stdcall zaLoadEntryByIndex(ZARCHIVE* Instance, const int idx, HGLOBAL* pBufHandle);
;   (ANSI)    char_t   __stdcall zaLoadEntryByIndex(ZARCHIVE* Instance, const int idx, HGLOBAL* pBufHandle);
;
; Description:
;
; Returns:
;   Filename of entry if idx is within limits of array
;   NULL if idx is not within limits of array
;
; ----------------------------------------------------------------------------
; variable offests into ESP upon function entering and adjustment of the
; stack for local variables and register preservation
; ----------------------------------------------------------------------------
; ESP + 0  = (return address)
; ESP + 4  = instance parameter
; ESP + 8  = idx parameter
; ESP + 12 = pbufHandle parameter
; ============================================================================
align 16
_zaLoadEntryByIndexCommon@12 proc public
    ; verify the 'idx' value is within array bounds limits
    mov eax, [esp + 4]
    mov edx, [esp + 8]    
    mov ecx, [eax].zaInstance.Directory

    ; check our lower bounds
    cmp edx, 0
    jb short OutofBounds
    
    ; check our upper bounds
    cmp edx, [edx].zdInstance.ArrayCount
    jge short OutofBounds
    
    ; get the ZENTRY object offset
    lea edx, [ecx + edx*4 + 4]

    ; call the actual loading routine now that we have the ZENTRY object
    push [esp + 12]     ; pBufHandle parameter
    push [eax].zaInstance.IO
    push [edx]
    call zaLoadEntry
    
    ret 12
    
OutofBounds:
    mov eax, 0
    ret 12
_zaLoadEntryByIndexCommon@12 endp

; ----------------------------------------------------------------------------
; prototype:
;   (UNICODE) wchar_t* __stdcall zaLoadEntryByNameCommon(ZARCHIVE* Instance, const wchar_t* pwszFilename, HGLOBAL* pBufHandle);
;   (ANSI)    char_t   __stdcall zaLoadEntryByNameCommon(ZARCHIVE* Instance, const char* pszFilename, HGLOBAL* pBufHandle);
;
; Description:
;
; Returns:
;   Filename of entry if idx is within limits of array
;   NULL if idx is not within limits of array
;
; ----------------------------------------------------------------------------
; variable offests into ESP upon function entering and adjustment of the
; stack for local variables and register preservation
; ----------------------------------------------------------------------------
; ESP + 0  = (return address)
; ESP + 4  = instance parameter
; ESP + 8  = pszFilename parameter
; ESP + 12 = pbufHandle parameter
; ============================================================================
align 16
_zaLoadEntryByNameCommon@12 proc
    ; get the ZENTRY object from the ZARCHIVE object
    mov edx, [esp + 4]
    mov ecx, [edx].zaInstance.Directory
    lea eax, [ecx].zdInstance.Array
    mov edx, [esp + 8]

    ; try and find the ZENTRY object based on the filename
    push offset flat:compareStringSearch
    push sizeof dword
    push [ecx].zdInstance.ArrayCount
    push eax
    push edx
    call BinarySearch
    
    ; did we find a match?
    test eax, eax
    je short FileNotFound
    
    ; load the entry now
    mov edx, [esp + 4]
    push [esp + 12]
    push [edx].zaInstance.IO
    push [eax]
    call zaLoadEntry
    
    ret 12
    
FileNotFound:
    mov eax, 0
    ret 12
_zaLoadEntryByNameCommon@12 endp

; these are here so we can have C/C++ callable functions with their
; correct ANSI/UNICODE string prototypes. If this library was only
; going to be called from ASM, we wouldn't need these.
IF _UNICODE eq 1
    _zaLoadEntryByIndexW proc public
        jmp zaLoadEntryByIndexCommon
    _zaLoadEntryByIndexW endp

    _zaLoadEntryByNameW@12 proc public
        jmp zaLoadEntryByNameCommon
    _zaLoadEntryByNameW@12 endp
ELSE
	include ansi\asm_Zip.inc
ENDIF
    _zaLoadEntryByIndexA@12 proc public
        jmp zaLoadEntryByIndexCommon
    _zaLoadEntryByIndexA@12 endp
    
    _zaLoadEntryByNameA@12 proc public
        jmp zaLoadEntryByNameCommon
    _zaLoadEntryByNameA@12 endp
END
