TITLE ""

.586
.model flat, stdcall      ; memory model & calling convention
option casemap :none      ; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
include Kernel32.inc


; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
include asm_Zip.inc


.code

; ----------------------------------------------------------------------------
; prototype:
; LPCTSTR __stdcall zaLoadEntry(ZENTRY* Instance, ZIO* Instance, HGLOBAL* pBufHandle);
; ----------------------------------------------------------------------------
align 16
zaLoadEntry proc uses ebx esi edi Instance:dword, IO:dword, BufHandle:dword
	LOCAL FileSize:dword
	LOCAL Result:dword
	
	mov	edi, [BufHandle]
	mov	ebx, [Instance]
	
	; make sure we have a valid zeEntry object pointer
	mov	dword ptr [edi], 0
	lea esi, [FileSize]
	test ebx, ebx
	je short @Failed

	; open up the ZENTRY file for decompressing, just sets up
	; memory pointers, etc.  There isn't a corresponding close
	; because nothing needs to be freed, etc.
	push esi
	push [IO]
	push ebx
	call zaOpenEntry
	test eax, eax		; zipNoError
	jne	short @Failed

	; if we dont' have a size, this is an empty item -- most
	; likely a folder
	cmp	dword ptr [esi], 0
	je short @Success
	
	; allocate our decompression buffer in a HGLOBAL handle
	; this allows the most flexible way of using the memory.
	; main reason it is GMEM_MOVABLE instead of GMEM_PTR is
	; because we can take this handle and directly pass it
	; to a GDI+ object for drawing (in case the buffer
	; is an image).
	;
	; Side effects require two additional calls to use this
	; handle for normal use.  GlobalLock and GlobalUnlock.
	; But in my opinion, it is worth it to have it this way.
	push [esi]
	push GMEM_MOVEABLE
	call GlobalAlloc
	test eax, eax		; make sure we have a buffer
	mov	[edi], eax
	je short @Failed
	
	; successful allocation, now decompress the zip entry
	; into this buffer with the GlobalLock, GlobalUnlock
	; wrapper calls
	push eax
	call GlobalLock
    push eax
    push [IO]
    push ebx
    call zaReadEntry
	mov	[Result], eax
	push [edi]
	call GlobalUnlock

	; make sure we decompressed the whole amount correctly
	mov ecx, [esi]
	mov	eax, [Result]
    cmp	eax, ecx
	je short @Success
	
	; free our buffer and return NULL for the filename
	push [edi]
	call GlobalFree
	
@Failed:
	xor	eax, eax
	ret	
	
@Success:
	; return our filename of this entry 
	mov	eax, [ebx].zeInstance.FileName
	ret
	
zaLoadEntry endp

END
