TITLE "Opens a zip format archive file"

.586
.model flat, stdcall      ; memory model & calling convention
option casemap :none      ; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
;include Windows.inc
include Kernel32.inc


; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
include asm_Memory.inc
include asm_zip.inc


.code
; ----------------------------------------------------------------------------
; prototype:
; int __stdcall zaOpenCommon(ZARCHIVE* Instance, HANDLE hFile);
;
; Description:
;
; NOTE:
;		The responsibility of freeing of the buffers created in this function
;		and zdOpenDirectory will be handled by zaCloseArchive.
;
; Returns:
;   Failed Open:    ZipBadZipFile
;   Another Value:  zaOpenArchiveCommon() returned value
;
; ----------------------------------------------------------------------------
; variable offests into ESP upon function entering and adjustment of the
; stack for local variables and register preservation
; ----------------------------------------------------------------------------
; ESP + 0 = (return address)
; ESP + 4 = instance parameter
; ESP + 8 = handle parameter
; ============================================================================
align 16
zaOpenCommon proc uses ebx esi Instance:dword, hFile:dword
	LOCAL ReadAmount:dword

	; get the size of the file
	push 0
	push [hFile]
	call GetFileSize
	cmp	eax, INVALID_FILE_SIZE
	je short Failure_BadFile

	; save our filesize
	mov	ebx, eax
	
	; create the instance data plus room to read the contents of the file into
	add	eax, 8
	
    push eax
    call fmAlloc
	test eax, eax
	je short Failure_Memory

	mov	esi, eax
	
	; start our memory pointer offset at the beginning of the buffer
	add	eax, 8
	mov	[esi].zfInstance.MemPosition, eax

	; save our filesize
	mov	[esi].zfInstance.MemBufSize, ebx
	
	; read in the file contents
	lea	eax, [ReadAmount]
	lea	ecx, [esi].zfInstance.MemBuffer
	push 0
	push eax
	push ebx
	push ecx
	push [hFile]
	call ReadFile
	test eax, eax
	je short Failure_BadFile
	
	; close our file handle
	push [hFile]
	call CloseHandle
	mov	ebx, [Instance]
	
	; save our opened file information in our instance object
	mov	[ebx].zaInstance.IO, esi
	
	; read in the central directory now and return the result
	lea	eax, [ebx].zaInstance.Directory
	push [ebx].zaInstance.IO
	push eax
	call zdOpenDirectory
	
	mov eax, zipNoError
	ret
	
Failure_BadFile:
    push [hFile]
	call CloseHandle
	mov	eax, ZipBadZipFile
	ret
	
Failure_Memory:
    push [hFile]
    call CloseHandle
	mov	eax, ZipMemError
	ret
zaOpenCommon endp

END
