TITLE ""

.586
.model flat, stdcall      ; memory model & calling convention
option casemap :none      ; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------


; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
include asm_Analysis.inc
include asm_Zip.inc


.code

; ----------------------------------------------------------------------------
; prototype:
; DWORD __stdcall zaReadEntry(ZENTRY* pEntry, LPVOID pBuf, DWORD dwSize);
; ----------------------------------------------------------------------------
align 4
zaReadEntry proc uses ebx esi edi Instance:dword, IO:dword, Buffer:dword
	mov	ecx, [Instance]
	mov	ebx, [ecx].zeInstance.CDHdr
	mov	edi, [Buffer]
	mov	edx, [ebx].CentralDirectoryHeader.Common.UnCompSize

	; see if the entry is compressed or not.
	movzx eax, word ptr [ebx].CentralDirectoryHeader.Common.Method
	test eax, eax		; no compression?
	jne	short @UnCompressIt

	; ---------------------
	; non-compressed entry
	; ---------------------
	
	; get a pointer to the uncompressed file memory contents
	mov	ecx, [IO]
	mov	esi, [ecx].zfInstance.MemPosition
	add	[ecx].zfInstance.MemPosition, edx

	; just do a straight copy of the zip entry contents from our file memory
	; buffer to the passed in buffer
	mov	edx, [ebx].CentralDirectoryHeader.Common.UnCompSize
	mov	ecx, edx
	shr	ecx, 2
	rep	movsd
	mov	ecx, edx
	and	ecx, 3
	rep	movsb

	mov	eax, edx
	ret
	
@UnCompressIt:
	; -----------------
	; compressed entry
	; -----------------

	; get a pointer to the compressed file memory contents and
	; move the current memory pointer to the new position
	mov	ecx, [IO]
	mov	edx, [ebx].CentralDirectoryHeader.Common.CompSize
	mov	eax, [ecx].zfInstance.MemPosition
	add	[ecx].zfInstance.MemPosition, edx
	
	; inflate (decompress it)
	push [ebx].CentralDirectoryHeader.Common.UnCompSize
	push edi
	push [ebx].CentralDirectoryHeader.Common.CompSize
	push eax
	call ZipInflate
	
	; validate and make sure the CRC32 values are the same now.
	xor ecx, ecx
	not ecx								; initialize the starting value of our crc - reflect value
	push eax
	push edi
	push ecx
	call Crc32
	not eax								; crc = ~crc - reflect value
	
	; compare the calculated CRC32 with the entry's crc32
	cmp	eax, [ebx].CentralDirectoryHeader.Common.Crc32
	jne	@F
	
	; success
	mov	eax, [ebx].CentralDirectoryHeader.Common.UnCompSize
	ret
	
@@:	; failure
	xor	eax, eax
	ret
zaReadEntry endp

END
